# -*- coding: utf-8 -*-
"""
Created on Mon May 30 11:02:09 2022

@author: Administrator
"""
import math
#point:x(lon),y(lat),t(tamp)
def Rad(d):
    return d * math.pi / 180

class Point:
    def __init__(self):
        self.lat = 0
        self.lon = 0
        self.time = 0
        self.id=0

def cacl_SED(pointS,pointE,pointM):
    radLonS = Rad(pointS.get_x())
    radLonM = Rad(pointM.get_x())
    radLonE = Rad(pointE.get_x())
    radLatS = Rad(pointS.get_y())
    radLatM = Rad(pointM.get_y())
    radLatE = Rad(pointE.get_y())
    numerator = pointM.get_time() - pointS.get_time()             #算tm-ts (分子)
    denominator = pointE.get_time() - pointS.get_time()           #算te-ts (分母)
    if(denominator==0):
        time_ratio =0
    else:
        time_ratio =numerator/denominator
    #分母（te-ts ）若为0，说明e,s两点时间戳t相同，则有te=ts=tm，则比值为1?否则计算分数的值
    lat = radLatS + (radLatE - radLatS)*time_ratio      #计算m’点的维度
    lon = radLonS + (radLonE - radLonS)*time_ratio   #计算m’点的经度
    lat_diff = lat - radLatM                   #计算m-m’的经度差
    lon_diff = lon - radLonM                #计算m-m’的经度差
    return math.sqrt(lat_diff*lat_diff + lon_diff*lon_diff)  #计算经纬度差的平方和的算术平方根


def TD_TR(point_list,output_point_list,Dmax):
    if len(point_list)<2:
        return 0
    start_index=0
    end_index=len(point_list)-1

    #起止点必定是关键点,但是作为递归程序此步引入了冗余数据,后期必须去除
    output_point_list.append(point_list[start_index])
    output_point_list.append(point_list[end_index])

    if start_index<end_index:
        index=start_index+1        #工作指针,遍历除起止点外的所有点
        max_vertical_dist=0        #路径中离弦最远的距离
        key_point_index=0        #路径中离弦最远的点,即划分点

        while(index<end_index):
            cur_vertical_dist=cacl_SED(point_list[start_index],point_list[end_index],point_list[index])
            if cur_vertical_dist>max_vertical_dist:
                max_vertical_dist=cur_vertical_dist
                key_point_index=index        #记录划分点
            index+=1

        #递归划分路径
        if max_vertical_dist>=Dmax:
            TD_TR(point_list[start_index:key_point_index],output_point_list,Dmax)
            TD_TR(point_list[key_point_index:end_index],output_point_list,Dmax)

#主程序
# point_list=[]
# output_point_list=[]

# #将待处理的数据写入
# fd=open(r"Sample.txt",'r')
# idx=0
# for line in fd:
#     line=line.strip()
#     longitude=float(line.split(" ")[0])
#     latitude=float(line.split(" ")[1])
#     time=float(line.split(" ")[2])
#     point_list.append((longitude,latitude,time,idx))
#     idx += 1  # did not use the point calss ,but just
# fd.close()


# import argparse
#
# parser = argparse.ArgumentParser()
# parser.add_argument("args", help="describe the args of the algs ,just for record ,need to change " \
#                                  "in the specific file, like '0.00045'")
# argps = parser.parse_args()
# Dmax = eval(argps.args)#0.00045#0.0000008#

def TD_TR_one_traj(point_list,Dmax):
    Poi_to_num={}
    for enu,i in enumerate(point_list):
        Poi_to_num[i]=enu
    output_point_list=[]
    TD_TR(point_list,output_point_list,Dmax=Dmax)

    output_point_list=list(set(output_point_list))        #去除递归引入的冗余数据
    output_point_list=sorted(output_point_list,key=lambda x:Poi_to_num[x])

    # 按照time排序,--但是后面面对重复的点需要单向处理
    # 好吧 这里的递归是完全不按照顺序的，考虑增加一个id变量来重新排序
    # print(output_point_list)
    return output_point_list

def TD_TR_batch(orig_list_of_Points,Dmax):
    compressed_list_of_Points=[]
    for traj in orig_list_of_Points:
        compressed_list_of_Points.append(TD_TR_one_traj(traj,Dmax))
    # print('TD_TR_batch ',len(orig_list_of_Points),' have done')
    return compressed_list_of_Points



# -*- coding: utf-8 -*-
"""
uniform batch form
"""
# class Point(object):
#     def __init__(self):
#         self.lat = 0
#         self.lon = 0
#         self.time = 0
#     def gpsreader(filename):         #读数据 存入points
#         file=open(filename,'r')
#         for n,line in enumerate(file):
#             if not line:
#                 break
#             li=line.split(" ")
#             point=Point()
#             point.lat =float(li[0])
#             point.lon = float(li[1])
#             point.time = float(li[2].strip())
#             points.append(point)
#         points.pop()
#         # print('reader:len(points)',len(points))
#         return

def Uniform(points,uniform):         #判断轨迹点是否应被保留，并返回保留点在Points中对应的下标
    simplified=[]             #下标保存
    originalIndex=0
    while(True):
        if originalIndex >= len(points)-1:
            break
        else:
            simplified.append(int(originalIndex))
            originalIndex = originalIndex + uniform## ?? why add a epsilon
    simplified.append(len(points)-1)
    simplified_points=[points[x] for x in simplified]
    return simplified_points

def Uniform_batch(orig_list_of_Points,uniform):
    compressed_list_of_Points=[]
    for traj in orig_list_of_Points:
        compressed_list_of_Points.append(Uniform(traj,uniform))
    print('Uniform_batch ',len(orig_list_of_Points),' have done')
    return compressed_list_of_Points

# points=[]                         #初始数据
# filename='Sample.txt'
#
# import argparse
#
# parser = argparse.ArgumentParser()
# parser.add_argument("args", help="describe the args of the algs ,just for record ,need to change " \
#                                  "in the specific file, like '5'")
# argps = parser.parse_args()
# epsilon = eval(argps.args)
#
# # epsilon=eval(input("epsilon please or default 5") or '5')      #键入阈值
# # epsilon=5                          #这里假设阈值为5
# # ##############################not a thershold at all! just a simple skip, skip 5 and choose one for example
# save_filename='Uniform_output.txt'
# Point.gpsreader(filename)
# cmp_buffer = Point.Uniform(epsilon)  #最终结果集合
# fd=open(save_filename,'w')          #写入并输出结果
#
# assert len(cmp_buffer)<=len(points) ,'seems reverse compressed'
#
# for i in range(len(cmp_buffer)):
#     fd.write("{} {} {}\n".format(points[cmp_buffer[i]].lat,points[cmp_buffer[i]].lon,points[cmp_buffer[i]].time))
# fd.close()




import math


class Point:
    def __init__(self, x:  float, y:  float, t:  float = None):
        """
        :param x: horizontal coordinate, e.g., lon
        :param y: vertical coordinate, e.g., lat
        :param t: time stamp
        """
        self._x = x
        self._y = y
        self._t = t
        self._cx = None
        self._cy = None
        self._cz = None

    def get_x(self) -> float:
        return self._x

    def get_y(self) -> float:
        return self._y

    def get_time(self) -> float:
        return self._t

    def get_cx(self) -> float:
        return self._cx

    def get_cy(self) -> float:
        return self._cy

    def get_cz(self) -> float:
        return self._cz

    def latlon2cartesian(self):
        """
        !! Only needed if self._x and self._y are lon and lat
        Convert lat-lon to x-y-z Cartesian coordinate system for distance computation
        Ref; https://en.wikipedia.org/wiki/Earth-centered,_Earth-fixed_coordinate_system
        """
        earth_r = 6371010  # unit: meter
        degree2radian = math.pi / 180  # Input lat/lon is decimal degree, while cos/sin requires radian

        self._cx = earth_r * math.cos(self._y * degree2radian) * math.cos(self._x * degree2radian)
        self._cy = earth_r * math.cos(self._y * degree2radian) * math.sin(self._x * degree2radian)
        self._cz = earth_r * math.sin(self._y * degree2radian)


#how to use this file :
# read txt from 'data' folder
#the code concering input and output is in line 299 and line 316
# txt is like "lat lon time\n" note the delimeter
import sys
import math
# sys.path.append("..")
#from test1 import test1
from ..basic_notion import DistFunc
from ..basic_notion.Point import Point
from ..basic_notion import LineSimplify
from .. import basic_notion as utils_copy

# import numpy as np



class DotPoint(Point):
    def __init__(self, x:  float, y:  float, t:  float, id: int):
        self._id = id
        super().__init__(x,y,t)
    def cals(self,sx,sx2,sy,sy2,sz,sz2,st,st2,sxt,syt,szt):
        sx += self.get_cx()
        sy += self.get_cy()
        sz += self.get_cz()
        st += self.get_time()
        sx2 += self.get_cx()**2
        sy2 += self.get_cy()**2
        sz2 += self.get_cz()**2
        st2 += self.get_time()**2
        sxt += self.get_cx()*self.get_time()
        syt += self.get_cy()*self.get_time()
        szt += self.get_cz()*self.get_time()

        self._sx = sx
        self._sy = sy
        self._sz = sz
        self._st = st

        self._sx2 = sx2
        self._sy2 = sy2
        self._sz2 = sz2
        self._st2 = st2

        self._sxt = sxt
        self._syt = syt
        self._szt = szt

        return sx, sx2, sy, sy2, sz, sz2, st, st2, sxt, syt, szt

    def get_sx(self) -> float:
        return self._sx

    def get_sx2(self) -> float:
        return self._sx2

    def get_sy(self) -> float:
        return self._sy

    def get_sy2(self) -> float:
        return self._sy2

    def get_sz(self) -> float:
        return self._sz

    def get_sz2(self) -> float:
        return self._sz2

    def get_st(self) -> float:
        return self._st

    def get_st2(self) -> float:
        return self._st2

    def get_sxt(self) -> float:
        return self._sxt

    def get_syt(self) -> float:
        return self._syt

    def get_szt(self) -> float:
        return self._szt

    def get_id(self) -> int:
        return self._id


# DOTS=DotsSimplify()


def addPoint(yi, n, pList):  # yi means epsilon # n* yi  is the bound #$ replace

    j = 0
    k = 0
    # $v=[[(0,-1,0)]]#v[k]=[(id,orderOfParent,point:(with property: x,y,time,id,cx,cy,cz,st,st2,sx,sx2,sy,sy2,sz,sz2,sxt,syt,szt))]#order指父节点在上一层的相对位置
    # 换成类 下面的参数全部都要改 plist[j].getbalabla   balabala

    sx, sx2, sy, sy2, sz, sz2, st, st2, sxt, syt, szt = (0 for i in range(11))
    sx, sx2, sy, sy2, sz, sz2, st, st2, sxt, syt, szt = pList[0].cals(sx, sx2, sy, sy2, sz, sz2, st, st2, sxt, syt, szt)
    v = [[(0, -1, pList[0])]]
    already = 0
    outPointList = [0]
    flag=0
    while j < len(pList) - 1:
        # 创建新的一层并填满
        k += 1  # k means the generation
        v.append([])  # v(k+1) a mepty gen
        termk = []  # record the order of term point
        #sx, sx2, sy, sy2, sz, sz2, st, st2, sxt, syt, szt = (0 for i in range(11))

        while j < len(pList) - 1:  # pointList# lon lat time id
            # 新进来的点编号j
            j += 1
            if flag==0:#if flag==0
                sx, sx2, sy, sy2, sz, sz2, st, st2, sxt, syt, szt = pList[j].cals(sx, sx2, sy, sy2, sz, sz2, st, st2, sxt,
                                                                              syt, szt)

            else:
                flag=0

            for i, pi in enumerate(v[k - 1]):

                if i in termk:
                    # print('termi,k',k,i,pi)
                    continue
                else:

                    p = LISSED(pi[2], pList[j])#easy_name
                    if p < yi:
                        v[k].append((pList[j].get_id(), i, pList[j]))  # $plist 直接给弄好 元素是dotpoint

                        break  # for
                    if p > n * yi:  # and n not in termk:origin the name need to be checked
                        termk.append(i)
                        #print('termk',termk)

            # if len(termk) >= len(v[k - 1]):  # 建好一层
            if len(v[k]) == 0 or len(termk) >= len(v[k - 1]):  # 建好一层

                if len(v[k]) == 0:
                    # if len(termk)==0:
                    #i = -1  # 其实这里应该考虑取最小的#￥
                    # print('len(termk)==0==v[k-1]')
                    i=min(range(len(v[k-1])),key=lambda x: LISSED(v[k-1][x][2],pList[j]))
                    v[k].append((pList[j].get_id(), i, pList[j]))
                #print('建好一层',v[k])
                v[k] = Minimize(v[k - 1], v[k])
                #print('建and 修好一层',v[k])
                List, already = TryToDecode(v, already)
                #print('List,already',List,already)
                outPointList.extend(List)
                j -= 1
                flag=1#if 1 this point need to be dismissed form sx
                break

    endlist = [v[k][-1][0]]
    paridx = v[k][-1][1]
    while k-1>already:
        paridx = v[k-1][paridx][1] #pList[endlist[-1]] is a DOTSPOINT obj
        # endlist.append(v[k-1][paridx][0])
        endlist.append(v[k - 1][min(len(v[k - 1]) - 1, paridx)][0])
        k-=1
    endlist.reverse()
    outPointList.extend(endlist)
    #print("outPointList,endlist")

    if outPointList[-1] != j:  # $here need the trackback
        outPointList.append(j)


    return outPointList


def TryToDecode(v, already):
    '''
    already 指之前已经完成输出的层的编号
    v也许可以考虑切片？
    '''
    List = []
    live = []

    for i in range(len(v) - already - 1):  # $
        live.append([0 for _ in range(len(v[i + already + 1]))])

    # live.append([1 for _ in range(len(v[-1]))])
    live[-1] = [1 for _ in range(len(live[-1]))]
    # print('try Decode,inilive',live)#$

    k = len(v) - already - 2
    d = 0
    for m in range(k, d, -1):  # m is the alyer of live
        for n, pj in enumerate(live[m]):
            if pj:
                live[m - 1][v[m + already + 1][n][1]] = 1
    m = d

    # print('try Decode,inilive2',live)#$

    def checkIfSingle(layer):
        sum = 0
        for n, i in enumerate(layer):
            sum += i
        if sum == 1:
            return True, n
        else:
            return False, -1

    m = already + 1
    while m < len(v):
        if not checkIfSingle(live[m - already - 1])[0]:  # m 是层数（对应的v
            break
        else:
            # print('checkIfSingle:live,m,already',live,m,already,checkIfSingle(live[m-already-1]))
            p = checkIfSingle(live[m - already - 1])[1]
            List.append(v[m][p][0])
            m += 1  # 很奇怪之前这里怎么跑的
    return List, already + len(List)

    # return outPointList, already


# termk=[]
def LISSED(i, j):#由于sx全部是在这个点之前的 所以 用还没有更新过的sx来计算

    if i.get_id()+1>=j.get_id():
        return 0
    else:
        eps= 1e-8
        eps2=1e-12
        c1 = (i.get_cx()*j.get_time()-j.get_cx()*i.get_time()+eps2)/(j.get_time()-i.get_time()+eps)
        c2 = (j.get_cx()-i.get_cx()+eps2)/(j.get_time()-i.get_time()+eps)
        c3 = (i.get_cy()*j.get_time()-j.get_cy()*i.get_time()+eps2)/(j.get_time()-i.get_time()+eps)
        c4 = (j.get_cy()-i.get_cy()+eps2)/(j.get_time()-i.get_time()+eps)
        c5 = (i.get_cz()*j.get_time()-j.get_cz()*i.get_time()+eps2)/(j.get_time()-i.get_time()+eps)
        c6 = (j.get_cz()-i.get_cz()+eps2)/(j.get_time()-i.get_time()+eps)

        dsx = j.get_sx()-j.get_cx()-i.get_sx()
        dsx2 = j.get_sx2() - j.get_cx()**2 - i.get_sx2()
        dsy = j.get_sy() - j.get_cy() - i.get_sy()
        dsy2 = j.get_sy2() - j.get_cy()**2 - i.get_sy2()
        dsz = j.get_sz() - j.get_cz() - i.get_sz()
        dsz2 = j.get_sz2() - j.get_cz()**2 - i.get_sz2()
        dsxt = j.get_sxt() - j.get_cx()*j.get_time() - i.get_sxt()
        dsyt = j.get_syt() - j.get_cy()*j.get_time() - i.get_syt()
        dszt = j.get_szt() - j.get_cz()*j.get_time() - i.get_szt()
        dst = j.get_st() - j.get_time() - i.get_st()
        dst2 = j.get_st2() - j.get_time()**2 - i.get_st2()
        LISSED = (c1**2+c3**2+c5**2)*(j.get_id()-i.get_id()-1)+\
            (c2**2+c4**2+c6**2)*(dst2)+\
             2*(c1*c2+c3*c4+c5*c6)*dst+\
             dsx2+dsy2+dsz2+\
            -2*c1*dsx-2*c3*dsy-2*c5*dsz+\
            -2*c2*dsxt-2*c4*dsyt-2*c6*dszt
        #print(i.get_id(),j.get_id(),LISSED)
    return LISSED


# def LISSED_DAG(i,j,pList):
#     sum=0
# #     if j-i==1:
# #         return sum
#     #a=time.time()
#     #print('LISSED',i,j)
#     for p in range(i+1,j):
#         sum+=getSed(pList[i],pList[j],pList[p])**2#$check the name
#     #print(time.time()-a)
#     return sum

def Minimize(par,child):#v[k]=[(idOfSelf,orderOfParent,dotspoint)]
    """
    不存具体的经纬度了 只存点位不知道好不好
    """
    for enu,i in enumerate(child):
        minLISSED=float("inf")#np.inf
        minParNum=0
        for n,j in enumerate(par):

            k=LISSED(j[2],i[2])
            if k<minLISSED:
                minParNum=n
                minLISSED=k

        # child[enu]=(i[0],minParNum,par[minParNum][2])
        i=(i[0],minParNum,par[minParNum][2])

    return child

# def formPoint(plist):
#     usefulList = []
#     # sx, sx2, sy, sy2, sz, sz2, st, st2, sxt, syt, szt = (0 for i in range(11))
#     # startTime = plist[0][2]
#     startTime=0
#     # last_time=0
#     for i in plist:
#         # if i[2]==last_time:
#         #     continue
#         # last_time=i[2]
#         a = DotPoint(i[0],i[1],i[2]-startTime,i[3])
#         a.latlon2cartesian()  # a have cx cy cz, t ,id  property
#         # sx+=a.get_cx()
#         # sx, sx2, sy, sy2, sz, sz2, st, st2, sxt, syt, szt = a.update(sx, sx2, sy, sy2, sz, sz2, st, st2, sxt, syt,
#         #                                                              szt)
#         usefulList.append(a)
#     return startTime,usefulList

# import utils_copy
def change_to_util_points(input_list):
    '''
    change current Point class to util class,
    list of points
    '''
    output_list=[]
    for i in input_list:
        util_point=utils_copy.Point.Point(i.get_x(),i.get_y(),i.get_time()) # pkg.module.class
        output_list.append(util_point)
    return output_list

def change_to_local_points(input_list):
    '''
    change util Point class to local Point class
    '''
    output_list=[]
    for enu,i in enumerate(input_list):
        local_point=DotPoint(i.get_x(),i.get_y(),i.get_time(),enu)
        local_point.latlon2cartesian()
        output_list.append(local_point)
    return output_list
####################################################main#############################################################

def out_pois(traj):
    pass
    # for i in traj:
    #     print(i.get_x(),i.get_y(),i.get_time())

flag=0
def DOTS_one_traj(pointList,arg_tup):
    global flag
    yi,n=arg_tup
    outNum = addPoint(yi=yi, n=n, pList=pointList) #like[0,3,5,12,16....]
    if flag==0:

        # print('L337')
        out_pois([pointList[i] for i in outNum])

    # print(yi,n,'outnum',outNum)
    outNum2=[]
    for i in outNum:
        if  i>0 and pointList[i].get_time()==pointList[i-1].get_time():
            print('duplicates!')
            continue
        if i > 0 and pointList[i].get_time() < pointList[i - 1].get_time():
            print('error!')
            continue
        outNum2.append(pointList[i]) # line [point[0],point[3],point[5],....]
    # outNum2 = [pointList[i] for i in outNum]
    outNum2=change_to_util_points(outNum2)
    if flag==0:
        # print('L348')
        out_pois(outNum2)
        flag=1

    return outNum2



def DOTS_batch(orig_list_of_Points, arg_tup):
    compressed_list_of_Points = []
    for enu,traj in enumerate(orig_list_of_Points):
        traj=change_to_local_points(traj)# also latlon2cartesian
        if enu==0:
            # print('input')
            out_pois(traj)
        compressed_list_of_Points.append(DOTS_one_traj(traj, arg_tup))
        if enu==0:
            # print('output')
            out_pois(compressed_list_of_Points[0])
            # 但是这里 输出的时候会少一点
    # print('DOTS_batch ',len(orig_list_of_Points),' have done')
    return compressed_list_of_Points

# pointList=[]
# data=open('Sample.txt','r')
# for n,line in enumerate(data):
#     lines=line.split(' ')
#     longitude=float(lines[0])
#     latitude=float(lines[1])
#     timef=float(lines[2].strip())
#     pointList.append((longitude,latitude,int(timef),n))
# startTime,pointList=formPoint(pointList)
# ####################################DOTS here addPoint means the DOTS algrithm
# # yi means epislon (the tolerance of ISSED )
# # n means  if Local SSED between new point and a typical point P is n times larger than episilon
# # then we think it is impossible that new point's parent point is before P
#
# import argparse
#
# parser = argparse.ArgumentParser()
# parser.add_argument("args", help="describe the args of the algs ,just for record ,need to change " \
#                                  "in the specific file, like '10,2'")
# argps = parser.parse_args()
# yi,n = eval(argps.args)
#
#
# outNum = addPoint(yi=yi, n=n, pList=pointList) #like[0,3,5,12,16....]
# outNum = [pointList[i] for i in outNum]#line [point[0],point[3],point[5],....]
#
# # print(f"compression rate is {round(len(outNum)/len(pointList),2)}")
#
# #####################################write file
# f = open("DOTS_output.txt", 'w')
# for i in range(len(outNum)):
#     if i>0 and outNum[i].get_time()==outNum[i-1].get_time():
#         # print('duplicates!')
#         continue
#     if i > 0 and outNum[i].get_time() < outNum[i - 1].get_time():
#         print('error!')
#         continue
#     f.write(str(outNum[i].get_x())+" ")
#     f.write(str(outNum[i].get_y())+" ")
#     f.write(str(outNum[i].get_time()+startTime)+" ")
#     f.write('\n')
# f.close()



# the following aims to check the lissed
# for i in range(10):
#     p=DotPoint(0,0,i,i)
#     p._cy=(i-5)*2
#     p._cx=(i%2)*2
#     p._cz=0
#     pointList.append(p)

# for i in pointList:
#
#     print('id cx cy cz time',i.get_id(),i.get_cx(),i.get_cy(),i.get_cz(),i.get_time())
#     if i.get_id()>20:
#         break
#print(pointList[0],pointList[0].get_time(),pointList[2].get_cx())


#outPointList = addPoint(1000,2,pointList)
# print(len(outPointList),outPointList)

########################################draw the picture################################################
#这里为了画图
# def get_MeanErr(point_list,output_point_list):
#     Err=0

#     start_index=0
#     end_index=len(output_point_list)-1

#     while(start_index<end_index):        #遍历所有关键点
#         #选取两相邻关键点
#         pointA_id=int(output_point_list[start_index].get_id())
#         pointB_id=int(output_point_list[start_index+1].get_id())

#         id=pointA_id+1        #工作指针,用于遍历非关键点
#         while(id<pointB_id):        #遍历两关键点之间的非关键点
#             Err+=DistFunc.synchronized_euclidean_distance(output_point_list[start_index],point_list[id],output_point_list[start_index+1])
#             id+=1

#         start_index+=1

#     return Err / len(point_list)


# def DOTS(yi,n,pointList):
#     outNum=addPoint(yi,n,pointList)
#     output_point_list=[pointList[i] for i in outNum]
#     get_MeanErr(pointList,output_point_list)
#     #print(len(output_point_list))
#     return len(output_point_list)/len(pointList),get_MeanErr(pointList,output_point_list)


# if __name__ == 'main':
#     x=list(range(200,2000,200))+list(range(2000,6000,500))
#     ERR=[]
#     compressionRate=[]
#     x=list(range(200,2000,200))+list(range(2000,6000,500))
#     for i in x:
#         k,w=DOTS(i,1.5,pointList)
#         compressionRate.append(k)
#         ERR.append(w)
#         #print(i)

#     print('DOTsPic')
#     import matplotlib.pyplot as plt
#     plt.subplot(2,1,1)
#     plt.plot(x,compressionRate,"b*")

#     plt.xlabel("epsilon")
#     plt.ylabel("CompressionRate")
#     plt.subplot(2,1,2)
#     plt.plot(x,ERR,"r-")
#     plt.xlabel("epsilon")
#     plt.ylabel("SED error")
#     #plt.legend()
#     plt.show()



# -*- coding: utf-8 -*-
"""
Created on Wed Jun 29 13:35:51 2022

@author: lenovo
"""
from .. import basic_notion as utils_copy
import math

class Point(object):
    def __init__(self,lat,lon,time):
        self.lat = lat
        self.lon = lon
        self.time = time

def cacl_angle(p1:Point,p2:Point):
    lat_diff = p2.lat - p1.lat
    lon_diff = p2.lon - p1.lon
    return math.atan2(lon_diff, lat_diff)

def safe_oriten(res_b:Point,res_c:Point,points_c:Point,points_d:Point,points_e:Point):
    angle_res_bc = cacl_angle(res_b, res_c)
    angle_de = cacl_angle(points_d, points_e)
    angle_res_bc_de = angle_de - angle_res_bc
    angle_points_cd = cacl_angle(points_c, points_d)
    angle_points_cd_de = angle_de - angle_points_cd
    if abs(angle_res_bc_de) > ori_threshold or abs(angle_points_cd_de) > ori_threshold:
        return False
    else:
        return True
    
def cacl_distance(p1:Point,p2:Point): 
    earth_r = 6371010
    degree2radian = math.pi / 180  # Assume the given lat/lon is in degree, while the cos/sin requires radian
    lat1, lon1 = p1.lat, p1.lon
    lat2, lon2 = p2.lat, p2.lon
    diff_lat, diff_lon = lat2 - lat1, lon2 - lon1
    a = (math.sin(diff_lat * degree2radian / 2))**2 + math.cos(lat1 * degree2radian) * math.cos(lat2 * degree2radian) * ((math.sin(diff_lon * degree2radian / 2))**2)
    c = 2 * math.atan2(math.sqrt(a), math.sqrt(1-a))
    return earth_r * c

def cacl_speed(a:Point,b:Point):
    return cacl_distance(a,b)/max(1e-6,(b.time-a.time))

def safe_speed(res_b:Point,res_c:Point,points_c:Point,points_d:Point,points_e:Point):
    res_speed=cacl_speed(res_b,res_c)
    points_speed=cacl_speed(points_c,points_d)
    max_res_distance = (points_e.time - points_d.time)*(res_speed + speed_threshold)
    min_res_distance = (points_e.time - points_d.time)*(res_speed - speed_threshold)
    max_points_distance = (points_e.time - points_d.time)*(points_speed + speed_threshold)
    min_points_distance = (points_e.time - points_d.time)*(points_speed - speed_threshold)
    distance_de = cacl_distance(points_d, points_e)
    if (distance_de <= max_res_distance and distance_de >= min_res_distance and distance_de <= max_points_distance and distance_de >= min_points_distance):
        return True
    else:
        return False

def threshold(points,threshold_tup):
    result=[]
    global speed_threshold,ori_threshold
    speed_threshold, ori_threshold=threshold_tup
    result.append(points[0])
    result.append(points[1])
    if len(points)>2:
        for i in range(2,len(points)-1): ## really careless ,zx!
            if(safe_speed(result[len(result)-2],result[len(result)-1],points[i-2],points[i-1],points[i])\
                and safe_oriten(result[len(result)-2],result[len(result)-1],points[i-2],points[i-1],points[i])):
                continue
            else:
                result.append(points[i])
        result.append(points[len(points)-1])# 最后一行没问题，但是前面的在len=2的时候，会重复
    elif len(points)==2:
        pass
        # result.append(points[len(points)-1])
    else:
        assert len(points)!=1,'len(points) should not be 1'
    return change_to_util_points(result)

def threshold_batch(orig_list_of_Points, threshold_tup):
    compressed_list_of_Points = []
    # orig_list_of_Points=list(map(change_to_local_points,compressed_list_of_Points))
    for traj in orig_list_of_Points:
        traj=change_to_local_points(traj)
        compressed_list_of_Points.append(threshold(traj, threshold_tup))
    # print('OPW_batch ',len(orig_list_of_Points),' have done')
    # return list(map(change_to_util_points,compressed_list_of_Points))
    return compressed_list_of_Points

# import utils_copy
def change_to_util_points(input_list):
    '''
    change current Point class to util class,
    list of points
    '''
    output_list=[]
    for i in input_list:
        util_point=utils_copy.Point.Point(i.lat,i.lon,i.time) # pkg.module.class
        output_list.append(util_point)
    return output_list

def change_to_local_points(input_list):
    '''
    change util Point class to local Point class
    '''
    output_list=[]
    for i in input_list:
        local_point=Point(i.get_x(),i.get_y(),i.get_time()) # pkg.module.class
        output_list.append(local_point)
    return output_list
# points=[]
# #print("请输入速度误差阈值、角度误差阈值，中间用逗号分割")
# # speed_threshold,ori_threshold = eval(input(
# #     "\请输入速度误差阈值、角度误差阈值，中间用逗号分割,enter to defalut '0.5,0.5" ) or '0.5,0.5')



import argparse
import unittest
from types import SimpleNamespace
import TrajCompress
import time
import pandas as pd



class TestTrajCompression(unittest.TestCase):

    def setUp(self):
        # 这里调用get_sample来获取测试数据
        self.orig_list_of_Points = TrajCompress.get_sample()
        assert len(self.orig_list_of_Points) > 0 and len(self.orig_list_of_Points[0]) > 0,\
            f'you should have at least one traj in Sample.txt'

    def test_compression(self):
        # parser = argparse.ArgumentParser()
        # parser.add_argument("--alg", default='Angular', help="name of the compression algorithm")
        # parser.add_argument("--args", default='3', help="algorithm specific arguments")
        # args = parser.parse_args()

        # config = SimpleNamespace(alg=args.alg, args=args.args)
        methods2args = {
            'OpeningWindow': '15',
            'Uniform': '5',
            'Dead_Reckoning': '2.9e-5',
            'TD_TR': '2e-6',
            'interval': '0.05',
            'Angular': '0.1',
            'bottom_up': '20',
            'threshold': '3,3',
            'STTrace': '50',
            'DP': '15',
            'DOTS': '1.2,0.8',
            'SQUISH_E': '2,1',
            'OPERB': '100'
        }
        results = []


        for name,args in methods2args.items():
            is_ordered=True
            config = SimpleNamespace(alg=name, args=args)


            start_time = time.time()
            compressed_points = TrajCompress.choose_compression_method(config, self.orig_list_of_Points)

            for i, o in zip(self.orig_list_of_Points, compressed_points):
                print(f"Original: {len(i)}, Compressed: {len(o)}")
                self.assertTrue(len(o) <= len(i))

            duration = time.time() - start_time
            for L_orig,L_comp in zip(self.orig_list_of_Points, compressed_points):
                if self._is_ordered_subsequence(L_orig,L_comp) is False:
                    is_ordered=False
                # self.assertTrue(self._is_ordered_subsequence(L_orig,L_comp),
                #                 f"Algorithm {name} failed to maintain order with args {args}")


            compression_rate = sum(len(o) for o in compressed_points) / sum(len(i) for i in self.orig_list_of_Points)
            results.append({
                'Algorithm': name,
                'Parameters': args,
                'Compression Rate': compression_rate,
                'Duration (seconds)': duration,
                'Is Ordered': is_ordered
            })
            print(f"{name} with args {args}: Duration = {duration:.2f} seconds, Is Ordered: {is_ordered}")



        # Write results to a CSV file
        with open('compression_results.csv', 'w', newline='') as file:
            result_df=pd.DataFrame(results)
            print(result_df)
            writer = result_df.to_csv(file, index=False)


        # check the points are in the same order, and they exist in the original traj
    @staticmethod
    def _is_ordered_subsequence(original, compressed):
        it = iter(original)
        last_time = None
        for comp_point in compressed:
            orig_point = next((x for x in it if abs(x.get_time() - comp_point.get_time())<1e-6), None)
            if orig_point is None:
                print(f"Point {comp_point} not found in original traj")
                return False  
            comp_time = comp_point.get_time()
            if last_time is not None and comp_time <= last_time:
                return False  # 如果时间不是递增的，返回False
            last_time = comp_time
        return True


        

if __name__ == '__main__':
    unittest.main()



'''
batch mode OpeningWindow,
'''
import math
from ..basic_notion.DistFunc import synchronized_euclidean_distance
from ..basic_notion.Point import Point
from ..basic_notion.LineSimplify import LineSimplify

points = []

def gpsreader(filename):
    with open(filename) as fin:
        lines = fin.readlines()
        for line in lines:
            data = line.strip().split()
            point = Point(float(data[0]), float(data[1]), float(data[2]))
            point.latlon2cartesian()
            points.append(point)

def gpsreader_batch(filename):
    '''
    goal from is
    [],[],[]
"TRIP_ID","CALL_TYPE","ORIGIN_CALL","ORIGIN_STAND","TAXI_ID","TIMESTAMP","DAY_TYPE","MISSING_DATA","POLYLINE"
"1372636858620000589","C","","","20000589","1372636858","A","False","[[-8.618643,41.141412],[-8.618499,41.141376],[-8.620326,41.14251],[-8.622153,41.143815],[-8.623953,41.144373],[-8.62668,41.144778],[-8.627373,41.144697],[-8.630226,41.14521],[-8.632746,41.14692],[-8.631738,41.148225],[-8.629938,41.150385],[-8.62911,41.151213],[-8.629128,41.15124],[-8.628786,41.152203],[-8.628687,41.152374],[-8.628759,41.152518],[-8.630838,41.15268],[-8.632323,41.153022],[-8.631144,41.154489],[-8.630829,41.154507],[-8.630829,41.154516],[-8.630829,41.154498],[-8.630838,41.154489]]"
"1372637303620000596","B","","7","20000596","1372637303","A","False","[[-8.639847,41.159826],[-8.640351,41.159871],[-8.642196,41.160114],[-8.644455,41.160492],[-8.646921,41.160951],[-8.649999,41.161491],[-8.653167,41.162031],[-8.656434,41.16258],[-8.660178,41.163192],[-8.663112,41.163687],[-8.666235,41.1642],[-8.669169,41.164704],[-8.670852,41.165136],[-8.670942,41.166576],[-8.66961,41.167962],[-8.668098,41.168988],[-8.66664,41.170005],[-8.665767,41.170635],[-8.66574,41.170671]]"
    '''
    with open(filename) as fin:
        lines = fin.readlines()
        for line in lines:
            data = line.strip().split()
            point = Point(float(data[0]), float(data[1]), float(data[2]))
            point.latlon2cartesian()
            points.append(point)


from timeout_decorator import timeout


# # @timeout(18000)
# def OPW(points,epsilon):
#     if len(points) < 3:
#         return points
#     # print(f'len(points),points',len(points),points)
#     print(f'len(points)',len(points))

#     for p in points:
#         p.latlon2cartesian()
#     e = 0
#     originalIndex = 0
#     simplified = [originalIndex]

#     e = originalIndex + 2
#     while e < len(points):
#         i = originalIndex + 1
#         condOPW = True
#         while i < e and condOPW:
#             if synchronized_euclidean_distance(points[originalIndex],
#                                                points[i], points[e]) > epsilon:
#                 condOPW = False
#             else:
#                 i += 1
#         if not condOPW:
#             originalIndex = i
#             simplified.append(originalIndex)
#             e = originalIndex + 2
#         else:
#             e += 1
#     if simplified[-1]!=len(points)-1:
#         simplified.append(len(points) - 1)
#     simplified_points=[points[x] for x in simplified]
#     return simplified_points


def OPW(points,epsilon):
    if len(points) < 3:
        return points
    # print(f'len(points),points',len(points),points)
    # print(f'len(points)',len(points))

    for p in points:
        p.latlon2cartesian()
    e = 0
    originalIndex = 0
    simplified = [originalIndex]

    e = originalIndex + 2
    while e < len(points):
        i = originalIndex + 1
        condOPW = True
        while i < e and condOPW:
            if synchronized_euclidean_distance(points[originalIndex],
                                               points[i], points[e]) > epsilon:
                condOPW = False
            else:
                i += 1
        if not condOPW:
            originalIndex = i
            simplified.append(originalIndex)
            e = originalIndex + 2
        else:
            e += 1
    if simplified[-1]!=len(points)-1:
        simplified.append(len(points) - 1)
    simplified_points=[points[x] for x in simplified]
    return simplified_points


def OPW_unlimit(points,epsilon):
    if len(points) < 3:
        return points
    # print(f'len(points),points',len(points),points)
    print(f'len(points)',len(points))

    for p in points:
        p.latlon2cartesian()
    e = 0
    originalIndex = 0
    simplified = [originalIndex]

    e = originalIndex + 2
    while e < len(points):
        i = originalIndex + 1
        condOPW = True
        while i < e and condOPW:
            if synchronized_euclidean_distance(points[originalIndex],
                                               points[i], points[e]) > epsilon:
                condOPW = False
            else:
                i += 1
        if not condOPW:
            originalIndex = i
            simplified.append(originalIndex)
            e = originalIndex + 2
        else:
            e += 1
    if simplified[-1]!=len(points)-1:
        simplified.append(len(points) - 1)
    simplified_points=[points[x] for x in simplified]
    return simplified_points



def OPW_detail(points,epsilon):
    if len(points) < 3:
        return points
    # print(f'len(points),points',len(points),points)
    print(f'len(points)',len(points))

    for p in points:
        p.latlon2cartesian()
    print(f'transform to cartesian done')
    e = 0
    originalIndex = 0
    simplified = [originalIndex]

    e = originalIndex + 2
    while e < len(points):
        i = originalIndex + 1
        condOPW = True
        while i < e and condOPW:
            if synchronized_euclidean_distance(points[originalIndex],
                                               points[i], points[e]) > epsilon:
                condOPW = False
                print(f'   L131,e:{e},i:{i}')

            else:
                i += 1
                print(f'L136 i:{i},e:{e}')

        if not condOPW:
            originalIndex = i
            simplified.append(originalIndex)
            e = originalIndex + 2
            print(f'L139 originalIndex:{originalIndex},e:{e}')
        else:
            e += 1
            print(f'L141 e:{e}')
    if simplified[-1]!=len(points)-1:
        simplified.append(len(points) - 1)
    simplified_points=[points[x] for x in simplified]
    return simplified_points



import tqdm

def OPW_batch(orig_list_of_Points,epsilon):
    compressed_list_of_Points=[]
    for traj in tqdm.tqdm(orig_list_of_Points):
        try:
            # compressed_list_of_Points.append(OPW(traj[:5000],epsilon))
            # compressed_list_of_Points.append(OPW(traj[:2000],epsilon))

            compressed_list_of_Points.append(OPW(traj,epsilon))

        except Exception as e:
            print('OPW_batch error:',e)
            import pdb
            pdb.set_trace()
    # print('OPW_batch ',len(orig_list_of_Points),' have done')
    return compressed_list_of_Points



if __name__ == '__main__':
    ## for I use command line to exec the py file,so the name will be __main__, if use import will be okay

    filename = 'Sample.txt'#input("Enter file name: ")
    epsilon = 8#float(input("Enter value of epsilon: "))
    save_filename = 'OpeningWindow_output.txt'#input("Enter name for output file: ")
    gpsreader(filename)
    cmp_index = OPW(points,epsilon)
    with open(save_filename, "w") as s_fp:
        for i in cmp_index:
            s_fp.write(f"{points[i].get_x()} {points[i].get_y()} {int(points[i].get_time())}\n")
        # s_fp.write(f"{timeuse / 1000000.0}\n")
    # print(epsilon,':',round(len(cmp_index)/len(points),2))

    # import argparse
    #
    # parser = argparse.ArgumentParser()
    # parser.add_argument("args", help="describe the args of the algs ,just for record ,need to change " \
    #                                  "in the specific file, like '0.0008'")
    # argps = parser.parse_args()
    # epsilon = eval(argps.args)
    # filename = 'Sample.txt'#input("Enter file name: ")
    # # epsilon = 0.0008#float(input("Enter value of epsilon: "))
    # save_filename = 'OpeningWindow_output.txt'#input("Enter name for output file: ")
    # gpsreader(filename)
    # cmp_index = OPW(points,epsilon)
    # with open(save_filename, "w") as s_fp:
    #     for i in cmp_index:
    #         s_fp.write(f"{points[i].get_x()} {points[i].get_y()} {int(points[i].get_time())}\n")
    # # print(epsilon,':',round(len(cmp_index)/len(points),2))















import os
import sys
import argparse
from types import SimpleNamespace

# def setup_environment():
#     """
#     Set up the environment for the script.
#     """
#     # Optionally, change the current working directory to the script directory
#     origional_dir = os.getcwd()
#     script_dir = os.path.dirname(os.path.abspath(__file__))
#     os.chdir(script_dir)

#     # Append necessary paths to sys.path
#     sys.path.append(os.path.join(script_dir,'..', '..', 'Zhou'))
#     sys.path.append(os.path.join(script_dir,'..', '..', 'Zhu'))
#     sys.path.append(os.path.join(script_dir,'..', '..', 'Liu'))

#     os.chdir(origional_dir)
#     # print('L22 after set',sys.path)

# def setup_environment_l2l():
#     """
#     Set up the environment for the script.
#     """
#     # Optionally, change the current working directory to the script directory
#     origional_dir = os.getcwd()
#     script_dir = os.path.dirname(os.path.abspath(__file__))
#     os.chdir(script_dir)

#     # Append necessary paths to sys.path
#     sys.path.append(os.path.join(script_dir,'..', '..', 'Zhou'))
#     sys.path.append(os.path.join(script_dir,'..', '..', 'Zhu'))
#     sys.path.append(os.path.join(script_dir,'..', '..', 'Liu'))

#     os.chdir(origional_dir)
#     print('L22 after set',sys.path)

def choose_compression_method(config, orig_list_of_Points):
    """
    Choose a compression method based on the given configuration.
    """
    # setup_environment_l2l()
    # print('l2l L45',sys.path)
    # import .algrithms.Angular_batch as Angular_b
    # import .algrithms.bottom_up_batch as bottom_up_b
    # import .algrithms.Dead_Reckoning_batch as Dead_Reckoning_b
    # import .algrithms.DP_batch as DP_b
    # import .algrithms.DOTS_batch as DOTS_b
    # import .algrithms.interval_batch as interval_b
    # import .algrithms.OpeningWindow_batch as OPW_b
    # import .algrithms.OPERB_batch as OPERB_b
    # import .algrithms.STTrace_batch as STTrace_b
    # import .algrithms.SQUISH_E_batch as SQUISH_E_b
    # import .algrithms.TD_TR_batch as TD_TR_b
    # import .algrithms.threshold_batch as threshold_b
    # import .algrithms.Uniform_batch as Uniform_b

    from .algorithms import Angular_batch as Angular_b
    from .algorithms import bottom_up_batch as bottom_up_b
    from .algorithms import Dead_Reckoning_batch as Dead_Reckoning_b
    from .algorithms import DP_batch as DP_b
    from .algorithms import DOTS_batch as DOTS_b
    from .algorithms import interval_batch as interval_b
    from .algorithms import OpeningWindow_batch as OPW_b
    from .algorithms import OPERB_batch as OPERB_b
    from .algorithms import STTrace_batch as STTrace_b
    from .algorithms import SQUISH_E_batch as SQUISH_E_b
    from .algorithms import TD_TR_batch as TD_TR_b
    from .algorithms import threshold_batch as threshold_b
    from .algorithms import Uniform_batch as Uniform_b
    
    alg = config.alg
    args = config.args
    implemented_algs=['OpeningWindow','Uniform','Dead_Reckoning','TD_TR','interval',
                  'Angular','bottom_up','threshold','STTrace','DP','DOTS','SQUISH_E',
                  'OPERB',]#'RLTS'
    assert alg in implemented_algs, f'{alg} not in the implemented_algs:{implemented_algs}'


    if alg == 'OpeningWindow':
        compressed_list_of_Points = OPW_b.OPW_batch(orig_list_of_Points, epsilon=float(args))
    if alg == 'Uniform':
        compressed_list_of_Points = Uniform_b.Uniform_batch(orig_list_of_Points, uniform=float(args))
    if alg == 'Dead_Reckoning':
        compressed_list_of_Points = Dead_Reckoning_b.Dead_Reckoning_batch(orig_list_of_Points, epsilon=float(args))
    if alg == 'TD_TR':
        compressed_list_of_Points = TD_TR_b.TD_TR_batch(orig_list_of_Points, Dmax=float(args))
    if alg == 'interval':
        compressed_list_of_Points = interval_b.interval_batch(orig_list_of_Points, epsilon=float(args))
    if alg == 'Angular':
        compressed_list_of_Points = Angular_b.Angular_batch(orig_list_of_Points, error_t=eval(args))
    if alg=='bottom_up':
        compressed_list_of_Points=bottom_up_b.bottom_up_batch(orig_list_of_Points,max_error=float(args))
    if alg=='threshold':
        compressed_list_of_Points=threshold_b.threshold_batch(orig_list_of_Points,threshold_tup=eval(args))
    if alg=='STTrace':
        compressed_list_of_Points=STTrace_b.STTrace_batch(orig_list_of_Points,cmp_ratio=int(float(args)))
    if alg=='DP':
        compressed_list_of_Points=DP_b.DP_batch(orig_list_of_Points,Dmax=float(args))
    if alg=='DOTS':
        compressed_list_of_Points=DOTS_b.DOTS_batch(orig_list_of_Points,arg_tup=eval(args))
    if alg=='SQUISH_E':
        compressed_list_of_Points=SQUISH_E_b.SQUISH_batch(orig_list_of_Points,arg_tup=eval(args))
    if alg=='OPERB':
        compressed_list_of_Points=OPERB_b.OPERB_batch(orig_list_of_Points,max_error=eval(args))
    if alg=='RLTS':

        print('this is not included in this package for it have much more dependencies, especailly about specific version of tensorflow')

        sys.path.append('/home/chenghao/RLTS')
        sys.path.append('/home/chenghao/RLTS/online-rlts')#TODO
        import list_Point_io
        import Point
        import os
        origional_dir = os.getcwd()
        os.chdir('/home/chenghao/RLTS/online-rlts')
        config.list_of_trajs=orig_list_of_Points#[:20000]#TODO
        
        compressed_trajectories=list_Point_io.run_trajectory_compression(config)
        compressed_list_of_Points=[[Point.Point(point_info[0],point_info[1],point_info[2])for point_info in sub_traj] for sub_traj in compressed_trajectories]
        # compressed_list_of_Points=[Point.Point(la,lo,time) for la,lo,time in compressed_trajectories]
        os.chdir(origional_dir)

    return compressed_list_of_Points

def get_sample(): #TODO, move this to utils
    # Get the directory where the script file is located
    # script_dir = os.path.dirname(os.path.abspath(__file__))
    # Setup environment
    # print('before',sys.path)
    # setup_environment_l2l()
    # print('after',sys.path)
    # Assume orig_list_of_Points is provided or loaded somewhere
    orig_list_of_Points = []  # Your list of points
    # Choose compression method
    import utils_copy
    this_traj=[]
    script_dir = os.path.dirname(os.path.abspath(__file__))
    file_path = os.path.join(script_dir, '..', '..', 'Liu', 'Sample.txt')
    with open(file_path) as f:
    # with open ('Sample.txt') as f:
        for line in f:
            a,o,t=list(map(eval,line.split()))
            this_traj.append(utils_copy.Point.Point(a,o,t))
    orig_list_of_Points.append(this_traj)
    orig_list_of_Points.append(this_traj[:20])
    return orig_list_of_Points

if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    # parser.add_argument("--alg",default='OPERB', help="name of the py file like 'bottom-up'")
    # parser.add_argument("--args",default='100',help="describe the args of the algs, just for record, like '90'")
    
    parser.add_argument("--alg",default='RLTS', help="name of the py file like 'bottom-up'")
    parser.add_argument("--args",default="/home/chenghao/RLTS/online-rlts/save_1211_06/1.4023512414981615e-05_ratio_0.6/",help="describe the args of the algs, just for record, like '90'")

    # parser.add_argument("--alg",default='SQUISH_E', help="name of the py file like 'bottom-up'")
    # # parser.add_argument("goal_folder",defalult='../Zhou', help="Path to input file like '../Zhou/'")
    # parser.add_argument("--args",default='1.2,2',help="describe the args of the algs, just for record, like '90'")
    

    # parser.add_argument("--alg",default='STTrace', help="name of the py file like 'bottom-up'")
    # # parser.add_argument("goal_folder",defalult='../Zhou', help="Path to input file like '../Zhou/'")
    # parser.add_argument("--args",default='54',help="describe the args of the algs, just for record, like '90'")
    

    args = parser.parse_args()

    # Convert argparse.Namespace to a simpler types.SimpleNamespace
    config = SimpleNamespace(alg=args.alg, args=args.args)
    # if RLTS:
    if not hasattr(config, 'ratio'):
        print(' you need to specify the compression ratio you want  (default 0.6)')
        config.ratio=float(input() or 0.6) 
    
    orig_list_of_Points=get_sample()
    # print(sys.path)
    compressed_points = choose_compression_method(config, orig_list_of_Points)
    # Do something with compressed_points...
    # assert compressed_points[0][0]
    for i,o in zip(orig_list_of_Points,compressed_points):
        print(len(i),len(o))


'''
python /home/user/ZCH/compression_GMVAE_STVEC/gmvae_compression_batch/0/l2l.py --alg OPERB --args 100

'''


#how to use this file :
# read txt from 'data' folder
# the code concerning input is line 223
# txt is like "lat lon time\n" note the delimeter
# the code concerning output is line 228
import sys
import math
#import ../Zhou/Sttrace
import heapq
#from heapq import _siftup, _siftdown, heapify
import bisect
import os
#from DOTS import get_MeanErr
# sys.path.append("..")
#from test1 import test1
from ..basic_notion.DistFunc import synchronized_euclidean_distance
from ..basic_notion.Point import Point
from ..basic_notion.LineSimplify import LineSimplify
from .. import basic_notion as utils_copy




class SQUPoint(Point):
    def __init__(self, x:  float, y:  float, t:  float, id: int,succ=None , pred=None,pi=0):
        self._id = id
        super().__init__(x,y,t)
        self._pred=pred
        self._succ=succ
        self._pi=pi
        self.weight=float('inf')
    def __lt__(self, other):
        return True
    def get_pred(self) :
        return self._pred

    def get_succ(self) :#$ check the -> code may be wrong
        return self._succ

    def get_id(self) -> int:
        return self._id
    def set_succ(self,succ):
        self._succ=succ
    def set_pred(self,pred):
        self._pred=pred

    def set_pi(self,pi):
        self._pi=pi

    def get_pi(self) -> int:
        return self._pi

class SQU_Alg(LineSimplify):#init_beta
    def __init__(self,init_beta=10,lam=10,miu=500):
        super().__init__()
        self.simplified_track=[]
        self.beta= init_beta
        self.lam = lam
        self.miu = miu
        self.q=[]
        self.startPoint=None
        self.tmp=None#this is made to save the latest point with inf weight
    def add_point(self, x: float, y: float, t: float = None, convert=True, id :int = None,succ = None,pred = None,pi:int = 0):
        """
        :param x: horizontal coordinate, e.g., lon
        :param y: vertical coordinate, e.g., lat
        :param t: time stamp
        :param convert: convert lat-lon to x-y-z Cartesian coordinate system -- needed if calculate distance later
        """
        p = SQUPoint(x, y, t,id,succ,pred,pi)
        #self, x:  float, y:  float, t:  float, id: int,succ=None , pred=None,pi=0):
        if convert:
            p.latlon2cartesian()

        self._trace.append(p)

    @staticmethod
    def set_priority(point,weight,q):
        # try:
        heapq.heappush(q, (weight, point))
        # except:
        #     heapq.heappush(q, (weight+0.01*k, point))
        #     k+=1

    def adjust_priority(self,point,q):#here is a little different from the persudo code
        # find the index of the point then update it and shift up(the inf weight point is handled in other way)
        if point.get_succ() and point.get_succ():
            p=point.get_pi()+synchronized_euclidean_distance(point.get_pred(),point,point.get_succ())
            #print("this weight ,p,len,point",point.weight,p,len(q),point,self.q)
            a=round(p,5)
            #i=bisect.bisect_left(q,(point.weight,point))
            i=q.index((point.weight,point))

            #print('find i',i)

            point.weight=a#I compare the weight in the bisect function  and use round to avoid the inaccuracy of float
            q[i]=(a,point)
            #print("suc i,len", i, len(q), self.q)
            if p>point.weight:
            #if  value is increased just shiftup the point
                heapq._siftup(q, i)
            elif p<point.weight:
                heapq._siftdown(q, 0, i)

            #self.set_priority(point,p,q)

    def reduce(self,q):
        #print("Before_reduce",len(q))
        _,pj=heapq.heappop(self.q)
        #print("after_pop", len(q))
        suc = pj.get_succ()
        pred = pj.get_pred()
        suc.set_pi(max(suc._pi, pj._pi))
        pred.set_pi(max(pred._pi, pj._pi))
        suc.set_pred(pred)
        pred.set_succ(suc)
        #remove the pj and make the pred and suc conncted
        if pred.get_pred():
            self.adjust_priority(pred, q)
        if pred.get_succ():
            self.adjust_priority(suc, q)

        #print("After_reduce", len(q))

    def Main_alg(self):
        inList = self.get_trace()
        for i in range(len(inList)):
            #print('i,beta',i,self.beta)
            if i/self.lam >=self.beta:
                self.beta+=1
            self.tmp=inList[i]
            if i > 1:
                a=synchronized_euclidean_distance(inList[i - 2], inList[i - 1], inList[i])
                self.set_priority(inList[i-1],round(a,5),self.q)
                inList[i - 1].weight = round(a,5)
            elif i==0:
                self.set_priority(inList[0],float('inf'),self.q)
                self.startPoint=inList[0]

                #if i==1 then do nothing  it is saved in tmp
            #here is different from the persudo code the article gives
            if i>=1:
                inList[i-1].set_succ(inList[i])
                inList[i].set_pred(inList[i-1])

                #self.adjust_priority(inList[i],self.q)

            if len(self.q)==self.beta:
                self.reduce(self.q)

        self.q.append((float('inf'),self.tmp))
        #print("####################################### the second stage")
        p,_=self.q[0]

        while p<self.miu:
            self.reduce(self.q)#$
            p=self.q[0][0]


        return self.form_trace()#in the priority queue lies points

    def form_trace(self):

        p=self.startPoint
        outList = [p]
        while p.get_succ():
            p=p.get_succ()
            outList.append(p)
        return outList


    def load_from_txt(self, filepath, time_include=False,delimter=","):
        if not os.path.exists(filepath):
            print("Please check if file exists AT ", filepath)
            exit()

        with open(filepath, "r") as rf:
            lines = rf.readlines()
            id = 0
            for line in lines:
                #id=0
                line = line.rstrip()
                if line and line[0] != '#':
                    if time_include:
                        lat, lon, ts = line.split(delimter)
                        self.add_point(float(lon), float(lat), float(ts),id=id)
                        id+=1
def get_MeanErr(point_list,output_point_list):
    Err=0.0
    start_index=0
    end_index=len(output_point_list)-1

    while(start_index<end_index):
        pointA_id=int(output_point_list[start_index].get_id())
        pointB_id=int(output_point_list[start_index+1].get_id())
        #print("first interval",pointA_id,pointB_id)
        id=pointA_id+1
        while(id<pointB_id):
            Err+=synchronized_euclidean_distance(output_point_list[start_index],point_list[id],output_point_list[start_index+1])
            id+=1
            #print("Err,id",Err,id)

        start_index+=1

    return Err / len(point_list)
        #return Err

# SQU_E = SQU_Alg(lam=1,miu=100)
# SQU_E.load_from_txt("..//data//test_with_time.txt",time_include=True,delimter=" ")
# #print('aaa', SQU_E._trace[90]._cx,SQU_E._trace[90]._x)
# outList = SQU_E.Main_alg()
#
# err=get_MeanErr(SQU_E.get_trace(),outList)
# print(len(outList),err)
# ol=[]
# for i in outList:
#     ol.append(i._id)




def SQU_Once(lam,miu,toFile=False):#

    SQU_E = SQU_Alg(lam=lam, miu=miu)
    SQU_E.load_from_txt("Sample.txt", time_include=True, delimter=" ")
    # print('load in', SQU_E._trace[90]._cx,SQU_E._trace[90]._x)
    outNum = SQU_E.Main_alg()
    # print(outNum)
    if toFile:
        f = open("SQUISH-E_output.txt", 'w')
        for i in outNum:
            f.write(str(i.get_y())+" ")
            f.write(str(i.get_x())+" ")
            f.write(str(i.get_time())+" ")
            f.write('\n')
        f.close()
    return len(outNum)/len(SQU_E._trace),get_MeanErr(SQU_E._trace,outNum)


def SQUISH_one_traj(pointList,arg_tup):
    lam,lam=arg_tup
    SQU_E = SQU_Alg(lam=lam, miu=lam)
    SQU_E._trace=pointList
    # change_to_local_points(pointList)
    outNum = SQU_E.Main_alg()
    return outNum


def SQUISH_batch(orig_list_of_Points, arg_tup):
    compressed_list_of_Points = []
    for traj in orig_list_of_Points:
        traj=change_to_local_points(traj)# also latlon2cartesian
        compressed_list_of_Points.append(SQUISH_one_traj(traj, arg_tup))
    # print('DOTS_batch ',len(orig_list_of_Points),' have done')
    return compressed_list_of_Points


# import utils_copy
def change_to_util_points(input_list):
    '''
    change current Point class to util class,
    list of points
    '''
    output_list=[]
    for i in input_list:
        util_point=utils_copy.Point.Point(i.lat,i.lon,i.time) # pkg.module.class
        output_list.append(util_point)
    return output_list

def change_to_local_points(input_list):
    '''
    change util Point class to local Point class
    '''
    output_list=[]
    for enu,i in enumerate(input_list):
        local_point=SQUPoint(i.get_x(),i.get_y(),i.get_time(),enu) # pkg.module.class
        local_point.latlon2cartesian()
        output_list.append(local_point)
    return output_list





# import argparse
#
# parser = argparse.ArgumentParser()
# parser.add_argument("args", help="describe the args of the algs ,just for record ,need to change " \
#                                  "in the specific file, like '5,10'")
# argps = parser.parse_args()
# lam,miu = eval(argps.args)
#
# SQU_Once(lam=lam,miu=miu,toFile=True)####the one that saves its output


# ########################################## to draw two plots to show the effect of the alg #####################################
# x=list(np.linspace(1,7,10))
# ERR=[]
# compressionRate=[]

# for i in x:
#     k,w=SQU_Once(i,0)
#     compressionRate.append(k)
#     ERR.append(w)


# import matplotlib.pyplot as plt
# plt.figure(figsize=(7,7))
# plt.subplot(2,1,1)
# plt.plot(x,compressionRate,"b*")
# plt.title('μ = 0 ; λ from 1 to 7 minimize SED error ensuring the compression ratio of λ')
# plt.xlabel("λ")
# plt.ylabel("CompressionRate")
# plt.subplot(2,1,2)
# plt.plot(x,ERR,"r-")
# plt.xlabel("λ")
# plt.ylabel("mean_SED error")
# #plt.legend()
# plt.show()



# x=list(np.linspace(10,300,10))
# ERR=[]
# compressionRate=[]

# for i in x:
#     k,w=SQU_Once(1,i)
#     compressionRate.append(k)
#     ERR.append(w)


# import matplotlib.pyplot as plt
# plt.figure(figsize=(7,7))
# plt.subplot(2,1,1)
# plt.plot(x,compressionRate,"b*")
# plt.title('λ = 1 ; μ from 10 to 300 maximizes compression ratio while keeping SED error under μ')
# plt.xlabel("μ")
# plt.ylabel("CompressionRate")
# plt.subplot(2,1,2)
# plt.plot(x,ERR,"r-")
# plt.xlabel("μ")
# plt.ylabel("mean_SED error")
# #plt.legend()
# plt.show()







# -*- coding: utf-8 -*-
"""
Created on Thu Jun  2 13:43:50 2022

@author: Administrator
"""
import math
# import utils_copy
from .. import basic_notion as utils_copy
#import numpy
class Interval:
    str="20081106231922"
    R=[]
    save_path=None
    averagerate=0
    class Point:
        def __init__(self):
            #instance fields found by Java to Python Converter:
            self.x = 0
            self.y = 0
            self.time = None
            self.num = 0
    #将角度约束在[0,2pi]之间
    def Standardization(a):
        if a<0:
            a=a+2*math.pi
        if a>=2*math.pi:
            a=a-2*math.pi
        return a 
    #定义区间
    class Range:
        def __init__(self):
            #instance fields found by Java to Python Converter:
            self.left = 0
            self.right = 0
    #给出一个角度的规范化后的扇形区域
    def Fan(d,error):
        range=Interval().Range()
        range.left=Interval.Standardization(d+error)
        range.right=Interval.Standardization(d-error)
        return range
    #判断两个区间是否有交集，True代表无交集
    def notInt(ComSub,range):
        if(ComSub.left<ComSub.right):  #ComSub的区间跨（0，1）方向
            if(range.left>range.right):
                if(range.right>ComSub.left and range.left<ComSub.right):
                    return True
        else:           #ComSub的区间不跨（0，1）方向
            if(range.left<range.right):
                if(ComSub.right>range.left and ComSub.left<range.right):
                    return True
            elif(range.left<ComSub.right or range.right>ComSub.left):
                return True
        return False
    #判断方向是否在区间内
    def isInt1(inter,direction):
        if inter.left<inter.right:
            if(direction<=inter.left)or(direction>=inter.right):
                return True
        else:
            if(direction<=inter.left)and(direction>=inter.right):
                return True
        return False
    #求交集（r1的范围小于r2）
    def Intersection(r1, r2):
        intersection=Interval.Range()
        if(r1.left<r1.right):#r1跨（0,1）
            if(r2.left<=r1.left)or(r2.left>=r1.right):#交叉
                intersection.left = r2.left
                intersection.right = r1.right
            elif((r2.right<=r1.left)or(r2.right>=r1.right)):#交叉
                intersection.left = r1.left
                intersection.right = r2.right
            else:#不交叉，r2包含了r1
                intersection.left = r1.left
                intersection.right = r1.right
        else:
            if((r2.left<=r1.left)and(r2.left>=r1.right)):
                intersection.left = r2.left
                intersection.right = r1.right 
            elif((r2.right<=r1.left)and(r2.right>=r1.right)):
                intersection.left = r1.left
                intersection.right = r2.right 
            else:
                intersection.left = r1.left
                intersection.right = r1.right 
        return intersection

def change_to_points(input_list):
    output_list=[]
    for i in input_list:
        util_point=utils_copy.Point.Point(i.x,i.y,i.time) # pkg.module.class
        output_list.append(util_point)
    return output_list
    # self.x = 0
    # self.y = 0
    # self.time = None
def one(Points_list,error=8):
    # error=math.pi*2.5#这里给定阈值为pi*2.5
    p=[]
    R=[]

    for i in Points_list:
        temppoint=Interval.Point()
        temppoint.x=i.get_x()
        temppoint.y=i.get_y()
        temppoint.time=i.get_time()
        p.append(temppoint)

    R.append(p[0])
    #计算p0p1方向
    direction12=Interval.Standardization(math.atan2(p[1].y-p[0].y, p[1].x-p[0].x))
    ComSub=Interval.Fan(direction12,error)
    s=0
    directsum=0
    for i in range(1,len(p)-1):
        direct=Interval.Standardization(math.atan2(p[i+1].y-p[i].y, p[i+1].x-p[0].x))
        interval=Interval.Fan(direct,error)
        if(Interval.notInt(ComSub, interval)==True):
            R.append(p[i])
            ComSub=interval
            s=i
        else:
            ComSub=Interval.Intersection(ComSub,interval)
            directsum=Interval.Standardization(math.atan2(p[i+1].y-p[s].y, p[i+1].x-p[s].x))
            if(Interval.isInt1(ComSub,directsum)==False):
                R.append(p[i])
                ComSub=interval
                s=i
    R.append(p[len(p)-1])
    T=len(p)
    T1=len(R)
    # print("compression rate:"+str((T1)/T))


    return change_to_points(R)

def interval_batch(orig_list_of_Points,epsilon):
    compressed_list_of_Points=[]
    for traj in orig_list_of_Points:
        compressed_list_of_Points.append(one(traj,epsilon))
    # print('OPW_batch ',len(orig_list_of_Points),' have done')
    return compressed_list_of_Points


# import argparse
#
# parser = argparse.ArgumentParser()
# parser.add_argument("args", help="describe the args of the algs ,just for record ,need to change " \
#                                  "in the specific file, like '7.85'(which is approximately the result of  3.1415926*2.5)")
# argps = parser.parse_args()
# error_parse = eval(argps.args)
#
# output_point_list=one(error=error_parse)
# fd=open(r"interval_output.txt",'w')
# for point in output_point_list:
#     fd.write("{} {} {}\n".format(point.x,point.y,point.time))
# fd.close()




from .src.List_Points_2_List_Points import choose_compression_method
from .utils.get_sample import get_sample


# from TrajCompress.src.List_Points_2_List_Points import choose_compression_method
# from TrajCompress.utils.get_sample import get_sample

# if __name__ == '__main__':
#     from types import SimpleNamespace
#     orig_list_of_Points = get_sample()
#     breakpoint()



#     parser.add_argument("--alg",default='OPERB', help="name of the py file like 'bottom-up'")
#     parser.add_argument("--args",default='100',help="describe the args of the algs, just for record, like '90'")
    
#     args = parser.parse_args()

#     # Convert argparse.Namespace to a simpler types.SimpleNamespace
#     config = SimpleNamespace(alg=args.alg, args=args.args)
    
#     print('orig_list_of_Points:', orig_list_of_Points)
#     choose_compression_method(config, orig_list_of_Points)
#     # Do something with compressed_points...
#     # assert compressed_points[0][0]
#     # parser.add_argument("--alg",default='RLTS', help="name of the py file like 'bottom-up'")
#     config = SimpleNamespace(alg=args.alg, args=args.args)

#     for i,o in zip(orig_list_of_Points,compressed_points):
#         print(len(i),len(o))


#     print('done')

#     '''
#     cd /home/chenghao/mk_pkg_compression/TrajCompress/
#     python __init__.py
#     '''


# -*- coding: utf-8 -*-
"""
Created on Thu Jun 30 17:48:15 2022

@author: Administrator
"""
import math

    
def cacl_distance(points):
    distance=[]
    for i in range(1, len(points)):
        distance.append(math.sqrt(math.pow(points[i].get_x()-points[i-1].get_x(),2)+math.pow(points[i].get_y() - points[i - 1].get_y(), 2)))
    return distance

def cacl_angle(points):
    anglee=[]
    for i in range(1,len(points)):
        lat_diff = points[i].get_x() - points[i-1].get_x()
        lon_diff = points[i].get_y() - points[i-1].get_y()
        anglee.append(math.atan2(lon_diff,lat_diff))
    return anglee

def Dead_Reckoning(points,eps):
    n=len(points)
    max_d = 0
    start_idx = 0
    d=cacl_distance(points)
    anglee=cacl_angle(points)
    simplifindex=[]
    simplifindex.append(0)
    for i in range(2, n):
        max_d = max_d + abs(d[i - 1] * math.sin(anglee[i - 1] - anglee[start_idx]))
        if abs(max_d)>eps:
            max_d =0
            simplifindex.append(i-1)
            start_idx=i-1
    if simplifindex[len(simplifindex)-1]!=n-1:
        simplifindex.append(n-1)
    simplified_points = [points[x] for x in simplifindex]
    return simplified_points

def Dead_Reckoning_batch(orig_list_of_Points,epsilon):
    compressed_list_of_Points=[]
    for traj in orig_list_of_Points:
        compressed_list_of_Points.append(Dead_Reckoning(traj,epsilon))
    print('Dead_Reckoning_batch ',len(orig_list_of_Points),' have done')
    return compressed_list_of_Points




from .Point import Point
import math


def perpendicular_euclidean_dist(start_point: Point, mid_point: Point, end_point: Point) -> float:
    """
    Step 1: Find the nearest point, p', of mid_point, on the line start_point - end_point
    Step 2: Calculate the Euclidean distance between mid_point and p'
    Note: The distance calculation is based on x-y-z Cartesian coordinate system --
          Please check Point.latlon2cartesian() before calling this function
                                  o (mid_point)
                                  |
        (start_point) o ----------o--- o (end_point)
                                  p'
    """
    start_x, start_y, start_z = start_point.get_cx(), start_point.get_cy(), start_point.get_cz()
    end_x, end_y, end_z = end_point.get_cx(), end_point.get_cy(), end_point.get_cz()
    mid_x, mid_y, mid_z = mid_point.get_cx(), mid_point.get_cy(), mid_point.get_cz()

    if any(val is None for val in [start_x, start_y, start_z, end_x, end_y, end_z, mid_x, mid_y, mid_z]):
        print("Distance calculation needs coordinate coversion from lat/lon to Cartesian one")
        exit()

    vec_start_mid = (mid_x - start_x, mid_y - start_y, mid_z - start_z)
    vec_start_end = (end_x - start_x, end_y - start_y, end_z - start_z)

    # dot_product_start2mid_start2end = |start-mid| |start-end| cos(\theta)
    # (\theta is the angle between start-mid and start-end)
    # norm_vec_start2end = |start-end| |start-end|
    dot_product_start2mid_start2end = sum(vec_start_mid[i] * vec_start_end[i] for i in range(3))
    norm_vec_start2end = sum(vec_start_end[i] ** 2 for i in range(3))

    if norm_vec_start2end == 0:
        print('Start and End points are the same one!')
        return 0#some instance we have object stay at one point for some time
        #exit()

    # project_ratio = |start-mid| cos(\theta) / |start-end|
    project_ratio = dot_product_start2mid_start2end / norm_vec_start2end

    # If project_ratio > 1, mid_point is outside the start-end line segment and the nearest point is simply p2
    # If project_ratio < 0, mid_point is outside the start-end line segment and the nearest point is simply p1
    project_ratio = max(0, min(1, project_ratio))

    projected_x = start_x + project_ratio * vec_start_end[0]
    projected_y = start_y + project_ratio * vec_start_end[1]
    projected_z = start_z + project_ratio * vec_start_end[2]

    return math.sqrt((projected_x - mid_x)**2 + (projected_y - mid_y)**2 + (projected_z - mid_z)**2)


def synchronized_euclidean_distance(start_point: Point, mid_point: Point, end_point: Point) -> float:
    """
    Step 1: Project mid_point to the line start_point - end_point based on Time Stamps
    Step 2: Calculate the Euclidean distance
    Note: The distance calculation is based on x-y-z Cartesian coordinate system --
          Please check Point.latlon2cartesian() before calling this function

                                  o (mid_point)
                                 /
                                /
        (start_point) o ------o------ o (end_point)
                               p' (based on time)
    """
    start_x, start_y, start_z = start_point.get_cx(), start_point.get_cy(), start_point.get_cz()
    end_x, end_y, end_z = end_point.get_cx(), end_point.get_cy(), end_point.get_cz()
    mid_x, mid_y, mid_z = mid_point.get_cx(), mid_point.get_cy(), mid_point.get_cz()

    if any(val is None for val in [start_x, start_y, start_z, end_x, end_y, end_z, mid_x, mid_y, mid_z]):
        print("Distance calculation needs coordinate coversion from lat/lon to Cartesian one")
        exit()

    start_t, end_t, mid_t = start_point.get_time(), end_point.get_time(), mid_point.get_time()

    if start_t is None or end_t is None or mid_t is None:
        print('Time stamp is missing from input!')
        exit()

    if start_t == end_t:
        return 0

    if start_t <= mid_t <= end_t:

        project_ratio = (mid_t - start_t) / (end_t - start_t)

        projected_x = start_x + project_ratio * (end_x - start_x)
        projected_y = start_y + project_ratio * (end_y - start_y)
        projected_z = start_z + project_ratio * (end_z - start_z)

        return math.sqrt((projected_x - mid_x) ** 2 + (projected_y - mid_y) ** 2 + (projected_z - mid_z) ** 2)
    else:
        print("Please check the time stamps of input!")
        print('start_point',start_point.get_x(),mid_point.get_x(),end_point.get_x())
        exit()


def haversine_distance(p1: Point, p2: Point) -> float:
    """
    https://www.movable-type.co.uk/scripts/latlong.html
    Given pairs of lat/lon position, return the distance based on Haversine formula
    """
    earth_r = 6371010
    degree2radian = math.pi / 180  # Assume the given lat/lon is in degree, while the cos/sin requires radian

    lat1, lon1 = p1.get_y(), p1.get_x()
    lat2, lon2 = p2.get_y(), p2.get_x()

    diff_lat, diff_lon = lat2 - lat1, lon2 - lon1

    a = (math.sin(diff_lat * degree2radian / 2))**2 + \
        math.cos(lat1 * degree2radian) * math.cos(lat2 * degree2radian) * ((math.sin(diff_lon * degree2radian / 2))**2)

    c = 2 * math.atan2(math.sqrt(a), math.sqrt(1-a))

    return earth_r * c









# -*- coding: utf-8 -*-
"""
Created on Thu Jun 30 18:41:38 2022

@author: Administrator
"""
import math
# import utils_copy
from .. import basic_notion as utils_copy

PI = 3.1415926


class Point(object):
    def __init__(self):
        self.lat = 0
        self.lon = 0
        self.time = 0
    # def gpsreader(filename):         #读数据 存入points
    #     file=open(filename,'r')
    #     for n,line in enumerate(file):
    #         if not line:
    #             break
    #         li=line.split(" ")
    #         point=Point()
    #         point.lat =float(li[0])
    #         point.lon = float(li[1])
    #         point.time = float(li[2].strip())
    #         P.append(point)
    #     # P.pop()#???? why pop --jijivski
    #     return

    def Standardization(a):
        if a<0:
            a=a+2*PI
        if a>=2*PI:
            a=a-2*PI
        return a

    def AngularDeviation(P,i):
        d1 = math.atan2(P[i].lon-P[i-1].lon,P[i].lat-P[i-1].lat)
        d2 = math.atan2(P[i+1].lon-P[i].lon,P[i+1].lat-P[i].lat)
        deviation = min(abs(d2-d1),2*PI-abs(d2-d1))
        if d1 >=0:
            if(d2>=d1+PI or d2<=d1):
                return -deviation
        else:
            if(d2>d1-PI and d2<d1):
                return -deviation
        return deviation

def Angular(points,error_t):# input is a list of util Points
    P = []
    R = []

    for i in points[1:]:
        temppoint = Point()
        temppoint.lon = i.get_x()
        temppoint.lat = i.get_y()
        temppoint.time = i.get_time()
        P.append(temppoint)


    R.append(0)
    deviation=0
    for i in range(1,len(P)-1):
        deviation = deviation +Point.AngularDeviation(P,i)
        if(abs(deviation)>error_t):
            R.append(i)
            deviation = 0

    R.append(len(P)-1)
    return [points[x] for x in R] # to util points

def change_to_points(input_list):
    output_list=[]
    for i in input_list:
        util_point=utils_copy.Point.Point(i.x,i.y,i.time) # pkg.module.class
        output_list.append(util_point)
    return output_list


def Angular_batch(orig_list_of_Points,error_t):
    compressed_list_of_Points=[]
    for traj in orig_list_of_Points:
        compressed_list_of_Points.append(Angular(traj,error_t))
    print('Angular_batch_batch ',len(orig_list_of_Points),' have done')
    return compressed_list_of_Points



# def fast_way(DUPLICATES):
#     return list(set(DUPLICATES))
def fast_way(DUPLICATES):
    return DUPLICATES
    # return list(set(DUPLICATES))


# error = PI/4
# error_t = error/2
# filename='Sample.txt'
# save_filename='Angular_output.txt'
#
# # with open(filename) as f:
# #     print(f.readline())
#     # print('f[0]',f[0])
#
# import argparse
# parser = argparse.ArgumentParser()
# parser.add_argument("args", help="describe the args of the algs ,just for record ,need to change " \
#                                  "in the specific file, like '3.1415926/4,3.1415926/8'")
# argps = parser.parse_args()
#
# # speed_threshold,ori_threshold = eval(argps.args)
# error,error_t=eval(argps.args)
#
# # print(error,error_t)
#

# print('haha',len(cmp_buffer))
# fd=open(save_filename,'w')          #写入并输出结果
# for i in range(len(cmp_buffer)):
#     fd.write("{} {} {}\n".format(P[cmp_buffer[i]].lat,P[cmp_buffer[i]].lon,P[cmp_buffer[i]].time))
# fd.close()



import math
# class Point(object):
#     def __init__(self):
#         self._x = 0
#         self._y = 0
#         self._t = 0

#把两个点变成一个有向线段
class GPSLine():				    
    def __init__(self,startPoint,endPoint):
        self.length=getDistanceOfP_P(startPoint._x, startPoint._y, endPoint._x, endPoint._y)
        self.angle=getAngleOfVector(startPoint._x, startPoint._y, endPoint._x, endPoint._y)             
    

def rad(d):
    return d * math.pi / 180
	
#两点距离	
def getDistanceOfP_P(lat1, lon1, lat2, lon2):#wei du ,jing du 			
	radLat1 = rad(lat1)
	radLat2 = rad(lat2)
	dw = radLat1 - radLat2 # 纬度
	dj = rad(lon1) - rad(lon2) 
	s = 2 * math.asin(math.sqrt(
				math.pow(math.sin(dw / 2), 2) + math.cos(radLat1) * math.cos(radLat2) * math.pow(math.sin(dj / 2), 2)))
	s = s * 6378137.0
	return s
    # checked wondering the lat and lon, the sequence here,
	

#有向线段的角度
import pdb

def getAngleOfVector(latS,  lonS,  latE,  lonE):
    try:
        if math.isclose(latS, latE, abs_tol=1e-8) and math.isclose(lonS, lonE, abs_tol=1e-8):
            return 0
        elif math.isclose(latS, latE, abs_tol=1e-8) and lonS < lonE:
            return 0
        elif math.isclose(latS, latE, abs_tol=1e-8) and lonS > lonE:
            return math.pi
        elif latS < latE and math.isclose(lonS, lonE, abs_tol=1e-8):
            return math.pi / 2
        elif latS > latE and math.isclose(lonS, lonE, abs_tol=1e-8):
            return 3.0 * math.pi/ 2
        high = getDistanceOfP_P(latE, lonE, latS, lonE)
        length = getDistanceOfP_P(latS, lonS, latE, lonE)
        if math.isclose(length, 0, abs_tol=1e-8):
            return 0
        
        angle = math.atan(high / length)       #in [0 pi/2]
        if latS > latE and lonS < lonE:
            angle = math.pi - angle          # in [pi/2, pi)
        if latS > latE and lonS > lonE:
            angle = math.pi + angle          #in [pi, 3pi/2)
        if latS < latE and lonS > lonE:
            angle = 2 * math.pi - angle      #in [3pi/2, 2pi)
        return angle
    except Exception as e:
        pdb.set_trace()

#点弦距离		
def getDistanceOfP_L(Rline, line):
    r = Rline.length
    angle1 = Rline.angle
    angle2 = line.angle
    if r == 0:
        return 0
    sita = getIncludedAngle(angle1, angle2)
    dis = r * math.sin(sita)
    return abs(dis)


#两个角度的差值
def getIncludedAngle( angle1, angle2):
    '''Included angles (∠). Given two directed line segments
    L1 =
    # » PsPe1 and L2 =
    # » PsPe2 with the same start point Ps,
    the included angle from L1 to L2, denoted as ∠(L1, L2), is
    L2.θ−L1.θ. For convenience, we also represent the included
    angle ∠(L1, L2) as ∠Pe1 PsPe2
    .'''
    theta = angle2 - angle1

    # if theta > math.pi:
	#     theta = -2 * math.pi + theta
    # if theta < -math.pi:
	#     theta = 2 * math.pi + theta       
    return theta

#Fitting Function中的f()函数
def f(Rline,lline):
    angle_diff=Rline.angle-lline.angle 
    if -2*math.pi<angle_diff and angle_diff<=-1.5*math.pi:
        return 1
    elif -math.pi<=angle_diff and angle_diff<=-0.5*math.pi:
        return 1
    elif  0<=angle_diff and angle_diff<=0.5*math.pi:
        return 1
    elif  math.pi<=angle_diff and angle_diff<1.5*math.pi:
        return 1
    else: 
        return -1
 

import copy
#fitting function函数F
def fittingFunction(points,start , i , lline , error):
    '''
    '''
    lline=copy.deepcopy(lline)
    Rline=GPSLine(points[start],points[i])
    j = int(Rline.length * 2/error - 0.5)+1
    value=j*error/2
    if Rline.length - lline.length<=error/4:
        lline=lline
    elif Rline.length>error/4 and lline.length==0:
        lline.length=value
        lline.angle=Rline.angle
    else:
        lline.length=value
        ascin_raw=getDistanceOfP_L(Rline,lline)/value
        if ascin_raw<=-1 or ascin_raw>=1:
            print('out of bound')
            ascin_raw = min(max(ascin_raw, -0.99), 0.99)
        # special variables
        # angle:1.2113205463421615
        # length:87.28782494224896
        # lline
        # <__main__.GPSLine object at 0x7f9690718b20>
        # special variables
        # angle:2.962579479887357
        # length:85.0
        # value
        # 85.0
        # maybe
        lline.angle=lline.angle+f(Rline,lline)*math.asin(ascin_raw)/j
    return lline
        
        
def getActivePoint(points,start,active,lline,error):
    '''
    逻辑是,根据一个圆 来判断是否在一个叫做active zone的范围内，得到不在这个范围内的点
    输入输出的目的是 下一个不再内的点（如果没点了就是-1） 然后就是是否能被加进去到当前的R的位置
    下一个active的点（如果没有了就是-1，然后这里是否能被归到前一个轨迹内）
    太抽象了，
    '''
    i=active+1
    if i>=len(points):
        return -1,False
    flag=True
    Rlinei=GPSLine(points[start],points[i])
    Rlinea=GPSLine(points[start],points[active])
    n=len(points)
    while Rlinei.length-lline.length<=error/4 and i<=n and i-start<4*100000:

        if getDistanceOfP_L(Rlinei,lline)>error/2 or getDistanceOfP_L(Rlinei,Rlinea)>error:
            flag=False
            # print(f'{i}th point be the new active')
            break
        i+=1
        if i>=len(points):
            return i,False
        # Lline.append(lline)# ?? what is it
        # print(f'{i}th point is is okay with the traj start {start} to {active}')
        Rlinei=GPSLine(points[start],points[i])
        # Rlinea=GPSLine(points[start],points[active])

        # 两种出去的情况： 
        # 1 if里面的break 就是说能fit，
        # 2 应该是上面while，主要的方法是 Rlinei.length-lline.length>error/4 超过这个圆 
    if getDistanceOfP_L(Rlinei,lline)>error/2 and lline.length>0:
        flag=False
    if i==len(points):
        i=-1  
    return i,flag

     
def OPERB(points,error):
    
    # global Lline
    Lline=[]

    res=[]   
    Lline.append(GPSLine(points[0],points[0]))
    end_bk=0
    end=0
    res.append(points[end])

    active,flag=getActivePoint(points,0,0,Lline[0],error)
    while active!=-1 and active<len(points)-1:
        # print('after add the new active:',active,end=' ')
        start=end 
        Lline.append(fittingFunction(points,start,active,Lline[len(Lline)-1],error))
        active,flag=getActivePoint(points,start,active,Lline[len(Lline)-1],error)
        while active!=-1 and active<len(points)-1 and flag==True:
            
            # 0.5 的时候直接跳过了这里的循环, 不断的添加0号点, active ++, flag=False
                # 需要更新？ 有进入循环的吗
            # 4的时候 直接 active -1 ，falg=True
                # 需要加入最后一个点
            Lline.append(fittingFunction(points,start,active,Lline[len(Lline)-1],error))
            end=active
            # print('L153')# this means the new active line contribute to the La, while the active and start 
            # print('inside new active:',active)


            # 61th point is is okay with the traj start 32 to 32
            # L153
            # inside new active: 61
            # 63th point is is okay with the traj start 32 to 61
            # 64th point is is okay with the traj start 32 to 61
            # 65th point is is okay with the traj start 32 to 61
            # L153
            # inside new active: 65
            # 67th point is is okay with the traj start 32 to 65
            # 68th point is is okay with the traj start 32 to 65
            # 69th point is is okay with the traj start 32 to 65
            # 70th point is is okay with the traj start 32 to 65
            # L153
            # inside new active: 70
            # 72th point is is okay with the traj start 32 to 70
            # 73th point is is okay with the traj start 32 to 70
            # 74th point is is okay with the traj start 32 to 70

            #from above the active Line La is changing, debug finished


            active,flag=getActivePoint(points,start,active,Lline[len(Lline)-1],error)
        
        if flag is False:
            end=active
            # if end==-1:
            #     end=
        
        # if end==end_bk:
        #     # active !=-1 but falg=False, can not fit in, so increase the end, 
        #     # meaning end is add to the compressed traj
        #     end+=1 
        end=min(end,len(points)-1)
        if points[end].get_time()<=res[-1].get_time():
            continue
        res.append(points[end])
        # print('res_id:',end)
        # end_bk=end  

    if end<len(points)-1:
        res.append(points[len(points)-1])

    return list(res)

    

# def test_GPSLine():
#     start = Point()
#     start._x, start._y = 0, 0
#     end = Point()
#     end._x, end._y = 1, 1
#     gps_line = GPSLine(start, end)
#     assert gps_line.length > 0, "Length should be positive"
#     assert 0 <= gps_line.angle <= 2 * math.pi, "Angle should be in range [0, 2π]"

# def test_getDistanceOfP_P():
#     assert getDistanceOfP_P(0, 0, 1, 1) > 0, "Distance should be positive"

# def test_getAngleOfVector():
#     assert 0 <= getAngleOfVector(0, 0, 1, 1) <= 2 * math.pi, "Angle should be in range [0, 2π]"

# Add more tests as needed
import tqdm


# if __name__=='__main__':
#     _run=True
#     if _run:
        # points=[]
        # Lline=[]
        # res=[]      
        # file=open('Sample.txt','r')    
        # for n,line in enumerate(file):
        #     if not line:
        #         break
        #     li=line.split(" ")
        #     point=Point()
        #     point._x =float(li[0])
        #     point._y = float(li[1])
        #     point._t = float(li[2].strip())
        #     points.append(point)
        # file.close()
import tqdm
def OPERB_batch(orig_list_of_Points,max_error):
    compressed_list_of_Points=[]
    for traj in tqdm.tqdm(orig_list_of_Points):
        compressed_list_of_Points.append(OPERB(traj,max_error))
    # print('OPW_batch ',len(orig_list_of_Points),' have done')
    return compressed_list_of_Points
        # print(f'points before compression {len(points)}')
        # print(f'points after compression {len(res)}')
        # fd=open(r"OPERBoutput.txt",'w')
        # for i in range(len(res)):
        #     fd.write("{} {} {}\n".format(res[i]._x,res[i]._y,res[i]._t))
        # fd.close()
        # print("compression ratio {}".format(len(res)/len(points)))


# Run tests
    # test_GPSLine()
    # test_getDistanceOfP_P()
    # test_getAngleOfVector()



# -*- coding: utf-8 -*-
"""
Created on Wed May 18 20:04:54 2022

@author: lenovo
"""

import math
import pickle

from .. import basic_notion as utils_copy

class Point(object):
    def __init__(self,lat,lon,time):
        self.lat = lat
        self.lon = lon
        self.time = time
    
        
class GPSPointWithSED(object):
    def __init__(self,point,sed):
        self.point = point
        self.sed=sed
        
#计算SED
def perpendicular_euclidean_dist(start_point: Point, mid_point: Point, end_point: Point):
    start_x,start_y,start_z = start_point.lat,start_point.lon,start_point.time
    end_x,end_y,end_z=end_point.lat,end_point.lon,end_point.time
    mid_x,mid_y,mid_z=mid_point.lat,mid_point.lon,mid_point.time	

    if any(val is None for val in [start_x, start_y, start_z, end_x, end_y, end_z, mid_x, mid_y, mid_z]):	
        print("Distance calculation needs coordinate coversion from lat/lon to Cartesian one") 
        exit()
    eps=1e-5
    vec_start_mid = (mid_x-start_x+eps , mid_y-start_y+eps , mid_z-start_z+eps)
    vec_start_end =(end_x-start_x+eps , end_y-start_y+eps , end_z-start_z+eps)

    if vec_start_end[0]==0 and vec_start_end[1]== 0:
        print('start and End points are the same one!')
        exit()	
	
    projected_x = start_x+ vec_start_mid[2] // vec_start_end[2] * vec_start_end[0]	
    projected_y = start_y+ vec_start_mid[2] // vec_start_end[2] * vec_start_end[1]	
    return math.sqrt((projected_x-mid_x)**2 + (projected_y -mid_y)**2)	


#需要注意的是：这里用户指定内存大小为cmp_ratio，但实际要多一个点的空间，这样才能计算buffer中最后一个点的SED
def STTrace(points,cmp_ratio):      #参数为用户指定的内存大小，这里简作为点的个数处理
    max_buffer_size = cmp_ratio  
    buffer = []                #buffer是最终结果集合
    buffer.append(GPSPointWithSED(points[0], 0))
   
    if max_buffer_size > 2:
        
        buffer.append(GPSPointWithSED(points[1], 0))
        for i in range(2,len(points)) :
            buffer.append(GPSPointWithSED(points[i], 0))
            # Compute SED for previous point
            segment_start = buffer[len(buffer) - 3].point
            segment_end = buffer[len(buffer) - 1].point
            buffer[len(buffer) - 2].sed = perpendicular_euclidean_dist(segment_start, buffer[len(buffer) - 2].point, segment_end)
            
         
            # Buffer full, remove a point
            if len(buffer) > max_buffer_size:   
                to_remove = max_buffer_size       #应删掉点的下标
                for buf in range(1,max_buffer_size):
                    if to_remove is max_buffer_size or buffer[buf].sed < buffer[to_remove].sed:
                        to_remove = buf
                if to_remove - 1 > 0:
                    buffer[to_remove - 1].sed = perpendicular_euclidean_dist(buffer[to_remove - 2].point, buffer[to_remove - 1].point, buffer[to_remove + 1].point)
                if to_remove + 1 < len(buffer) - 1:
                    buffer[to_remove + 1].sed = perpendicular_euclidean_dist(buffer[to_remove - 1].point, buffer[to_remove + 1].point, buffer[to_remove + 2].point)
                del buffer[to_remove]
        # i += 1#why +=1 ? what is i?
    else:
        buffer.append(GPSPointWithSED(points[len(points) - 1], 0))
    return change_to_util_points([i.point for i in buffer])


def STTrace_batch(orig_list_of_Points,cmp_ratio):
    compressed_list_of_Points=[]
    for traj in orig_list_of_Points:
        traj=change_to_local_points(traj)
        compressed_list_of_Points.append(STTrace(traj,cmp_ratio))
    # print('OPW_batch ',len(orig_list_of_Points),' have done')
    return compressed_list_of_Points


# import utils_copy
def change_to_util_points(input_list):
    '''
    change current Point class to util class,
    list of points
    '''
    output_list=[]
    for i in input_list:
        util_point=utils_copy.Point.Point(i.lat,i.lon,i.time) # pkg.module.class
        output_list.append(util_point)
    return output_list

def change_to_local_points(input_list):
    '''
    change util Point class to local Point class
    '''
    output_list=[]
    for i in input_list:
        local_point=Point(i.get_x(),i.get_y(),i.get_time()) # pkg.module.class
        output_list.append(local_point)
    return output_list

# #主程序
# # print('主程序')
# points=[]         #原数据集合
# file=open('Sample.txt','r')    #读数据 存入points
# for n,line in enumerate(file):
#     if not line:
#         break
#     li=line.split(" ")
#     point=Point()
#     point.lat =float(li[0])
#     point.lon = float(li[1])
#     point.time = float(li[2].strip())
#     points.append(point)
#
# # cmp_ratio = eval(input('number of points reserved (default 20: just enter)') or '20')     #用户指定的内存大小约束
# import argparse
#
# parser = argparse.ArgumentParser()
# parser.add_argument("args", help="describe the args of the algs ,just for record ,need to change " \
#                                  "in the specific file, like '20'")
# argps = parser.parse_args()
# cmp_ratio = int(eval(argps.args))#150#$#eval(input('max_error default 0.7') or '0.7')
# # print(max_error)
#
# cmp_buffer = STTrace(cmp_ratio)  #最终结果集合
# # print('line 101!!!!!!!!!')
# fd=open(r"STTrace_output.txt",'w')
# for i in range(len(cmp_buffer)):
#     fd.write("{} {} {}\n".format(cmp_buffer[i].point.lat,cmp_buffer[i].point.lon,cmp_buffer[i].point.time))
# fd.close()




# -*- coding: utf-8 -*-
"""
Created on Sat May 21 17:05:52 2022

@author: lenovo
"""

import math

def Rad(d):
    return d * math.pi / 180

def Geodist(point1,point2):
    radLat1 = Rad(point1.get_y())
    radLat2 = Rad(point2.get_y())
    delta_lon = Rad(point1.get_x()- point2.get_x())
    top_1 = math.cos(radLat2) * math.sin(delta_lon)
    top_2 = math.cos(radLat1) * math.sin(radLat2) - math.sin(radLat1) * math.cos(radLat2) * math.cos(delta_lon)
    top = math.sqrt(top_1 * top_1 + top_2 * top_2)
    bottom = math.sin(radLat1) * math.sin(radLat2) + math.cos(radLat1) * math.cos(radLat2) * math.cos(delta_lon)
    delta_sigma = math.atan2(top, bottom)
    distance = delta_sigma * 6378137.0

    return round(distance,3)

#点弦距离
def get_vertical_dist(pointA,pointB,pointX):
    a=math.fabs(Geodist(pointA,pointB))

    #当弦两端重合时,点到弦的距离变为点间距离
    if a==0:
        return math.fabs(Geodist(pointA,pointX))

    b=math.fabs(Geodist(pointA,pointX))
    c=math.fabs(Geodist(pointB,pointX))
    p=(a+b+c)/2
    S=math.sqrt(math.fabs(p*(p-a)*(p-b)*(p-c)))

    vertical_dist=S*2/a

    return vertical_dist

#递归压缩
def DP_compress(point_list,output_point_list,Dmax):
    start_index=0
    end_index=len(point_list)-1
    assert len(point_list)>1, f'len(point_list) is {len(point_list)},'
    output_point_list.append(point_list[start_index])
    output_point_list.append(point_list[end_index])

    if start_index<end_index:
        index=start_index+1        #工作指针,遍历除起止点外的所有点
        max_vertical_dist=0        #路径中离弦最远的距离
        key_point_index=0        #路径中离弦最远的点,即划分点

        while(index<end_index):
            cur_vertical_dist=get_vertical_dist(point_list[start_index],point_list[end_index],point_list[index])
            if cur_vertical_dist>max_vertical_dist:
                max_vertical_dist=cur_vertical_dist
                key_point_index=index        #记录划分点
            index+=1

        #递归划分路径
        if max_vertical_dist>=Dmax:
            if start_index+1<key_point_index:
                DP_compress(point_list[start_index:key_point_index],output_point_list,Dmax)
            if key_point_index + 1 < end_index:
                DP_compress(point_list[key_point_index:end_index],output_point_list,Dmax)


def DP_one_traj(point_list,Dmax):
    Poi_to_num={}
    for enu,i in enumerate(point_list):
        Poi_to_num[i]=enu
    output_point_list=[]
    DP_compress(point_list,output_point_list,Dmax=Dmax)

    output_point_list=list(set(output_point_list))        #去除递归引入的冗余数据
    output_point_list=sorted(output_point_list,key=lambda x:Poi_to_num[x])

    # 按照time排序,--但是后面面对重复的点需要单向处理
    # 好吧 这里的递归是完全不按照顺序的，考虑增加一个id变量来重新排序
    # print(output_point_list)
    return output_point_list

def DP_batch(orig_list_of_Points,Dmax):
    compressed_list_of_Points=[]
    for traj in orig_list_of_Points:
        compressed_list_of_Points.append(DP_one_traj(traj,Dmax))
    # print('TD_TR_batch ',len(orig_list_of_Points),' have done')
    return compressed_list_of_Points


# #平均误差
# def get_MeanErr(point_list,output_point_list):
#     Err=0
#
#     start_index=0
#     end_index=len(output_point_list)-1
#
#     while(start_index<end_index):        #遍历所有关键点
#         #选取两相邻关键点
#         pointA_id=int(output_point_list[start_index][2])
#         pointB_id=int(output_point_list[start_index+1][2])
#
#         id=pointA_id+1        #工作指针,用于遍历非关键点
#         while(id<pointB_id):        #遍历两关键点之间的非关键点
#             Err+=get_vertical_dist(output_point_list[start_index],output_point_list[start_index+1],point_list[id])
#             id+=1
#
#         start_index+=1
#
#     return Err/len(point_list)

#主程序
# point_list=[]
# output_point_list=[]
# times=[]
# #将处理后的数据写入内存
# fd=open(r"Sample.txt",'r')
# id=0
# for line in fd:
#     line=line.strip()
#     # id=int(line.split(",")[0])
#
#     longitude=float(line.split(" ")[0])
#     latitude=float(line.split(" ")[1])
#     point_list.append((longitude,latitude,id))
#     id += 1
#     time=float(line.split(" ")[2])
#     times.append(int(float(time)))
# fd.close()
#
# import argparse
#
# parser = argparse.ArgumentParser()
# parser.add_argument("args", help="describe the args of the algs ,just for record ,need to change " \
#                                  "in the specific file, like '8'")
# argps = parser.parse_args()
# # speed_threshold,ori_threshold = eval(argps.args)
# Dmax=int(eval(argps.args))
#
# DP_compress(point_list,output_point_list,Dmax=Dmax)
#
# output_point_list=list(set(output_point_list))        #去除递归引入的冗余数据
# output_point_list=sorted(output_point_list,key=lambda x:x[2])        #按照id排序
#
# #将压缩数据写入输出文件
# fd=open(r"DP_output.txt",'w')
# for point in output_point_list:
#     fd.write("{} {} {}\n".format(point[0],point[1],times[point[2]]))
# fd.close()

# print("compression rate={}/{}={}".format(len(point_list),len(output_point_list),len(output_point_list)/len(point_list)))
# print("mean error:{}".format(get_MeanErr(point_list,output_point_list)))



# import DistFunc
from .Point import Point
import os.path


class LineSimplify:
    def __init__(self):
        self._trace = []

    def add_point(self, x: float, y: float, t: float = None, convert=True):
        """
        :param x: horizontal coordinate, e.g., lon
        :param y: vertical coordinate, e.g., lat
        :param t: time stamp
        :param convert: convert lat-lon to x-y-z Cartesian coordinate system -- needed if calculate distance later
        """
        p = Point(x, y, t)

        if convert:
            p.latlon2cartesian()

        self._trace.append(p)

    def bunch_add_points(self, xs: list, ys: list, ts: list = None, convert=True):
        """
        :param xs: a list of horizontal coordinates, e.g., lon
        :param ys: a list of vertical coordinates, e.g., lat
        :param ts: a list of time stamps
        :param convert: convert lat-lon to x-y-z Cartesian coordinate system -- needed if calculate distance later
        """
        if ts is None:
            ts = [None] * len(xs)

        if min([len(xs), len(ys), len(ts)]) == 0:
            print("Please check the inputs -- some of them is empty")
            exit()

        len_input = len(xs)

        if any(len_input != len(input_l) for input_l in [ys, ts]):
            print("Please check the inputs -- some of them has different length")
            exit()

        for idx in range(len_input):
            p = Point(xs[idx], ys[idx], ts[idx])

            if convert:
                p.latlon2cartesian()

            self._trace.append(p)

    def load_from_txt(self, filepath, time_include=False,delimter=","):
        if not os.path.exists(filepath):
            print("Please check if file exists AT ", filepath)
            exit()

        with open(filepath, "r") as rf:
            lines = rf.readlines()

            for line in lines:
                line = line.rstrip()
                if line and line[0] != '#':
                    if time_include:
                        lat, lon, ts = line.split(delimter)
                        self.add_point(float(lon), float(lat), float(ts))
                    else:
                        lat, lon = line.split(delimter)
                        self.add_point(float(lon), float(lat))

    def get_trace(self):
        return self._trace

    def DouglasPeuckerAlgorithm(self):
        """
        Some Simplification Algorithm
        :return:
        """
        ### NEED TO BE FILLED ###
        pass







from setuptools import setup, find_packages

setup(
    name="TrajCompress",
    version="0.1.5",
    # packages=find_packages('TrajCompress'),
    packages=find_packages(),
    install_requires=[
        # 'argparse',  
        # 'os',
        # 'math',
        'pandas',
        'tqdm',
    ],
    author='jijivski',
    author_email='jijivski@outlook.com',
    url='https://github.com/jijivski/TrajCompress',  # URL to the repository
)


#python -m pip install --upgrade build


# -*- coding: utf-8 -*-
"""
Created on Thu Jun 30 12:02:31 2022

@author: lenovo
"""


import math
import os
import sys

from ..basic_notion import Point

class GPSPointWithSED(object):
    def __init__(self,point,sed):
        self.point = point
        self.sed=sed

def synchronized_euclidean_distance(start_point: Point, mid_point: Point, end_point: Point) -> float:

    
    start_x, start_y, start_z = start_point.get_cx(), start_point.get_cy(), start_point.get_cz()
    end_x, end_y, end_z = end_point.get_cx(), end_point.get_cy(), end_point.get_cz()
    mid_x, mid_y, mid_z = mid_point.get_cx(), mid_point.get_cy(), mid_point.get_cz()

    if any(val is None for val in [start_x, start_y, start_z, end_x, end_y, end_z, mid_x, mid_y, mid_z]):
        print("Distance calculation needs coordinate coversion from lat/lon to Cartesian one")
        sys.exit()

    start_t, end_t, mid_t = start_point.get_time(), end_point.get_time(), mid_point.get_time()

    if start_t is None or end_t is None or mid_t is None:
        print('Time stamp is missing from input!')
        sys.exit()

    if start_t <= mid_t <= end_t:
        project_ratio = (mid_t - start_t) / (end_t - start_t+0.0000001)

        projected_x = start_x + project_ratio * (end_x - start_x)
        projected_y = start_y + project_ratio * (end_y - start_y)
        projected_z = start_z + project_ratio * (end_z - start_z)

        return math.sqrt((projected_x - mid_x) ** 2 + (projected_y - mid_y) ** 2 + (projected_z - mid_z) ** 2)
    else:
        print("Please check the time stamps of input!")
        sys.exit()

def bottom_up(points,max_error):
    for p in points:
        p.latlon2cartesian()

    res=[]
    res.append(GPSPointWithSED(points[0], float('inf')))     
    res.append(GPSPointWithSED(points[1], 0))
    for i in range(2,len(points)) :
        res.append(GPSPointWithSED(points[i], 0))
        # Compute SED for previous point
        segment_start = res[len(res) - 3].point
        segment_end = res[len(res)-1].point
        res[len(res) - 2].sed = synchronized_euclidean_distance(segment_start, res[len(res) - 2].point, segment_end)
    res.append(GPSPointWithSED(points[len(points)-1], float('inf')))
    while True:
        to_remove = 1    
        for buf in range(1,len(res)-1):
            if res[buf].sed < res[to_remove].sed:
               to_remove = buf
        if res[to_remove].sed<=max_error:
            if to_remove - 1 > 0:
                res[to_remove - 1].sed = synchronized_euclidean_distance(res[to_remove - 2].point, res[to_remove - 1].point, res[to_remove + 1].point)
            if to_remove + 1 < len(res) - 1:
                res[to_remove + 1].sed = synchronized_euclidean_distance(res[to_remove - 1].point, res[to_remove + 1].point, res[to_remove + 2].point)
            del res[to_remove]
        else:
            break
    return [i.point for i in res]


def bottom_up_batch(orig_list_of_Points,max_error):
    compressed_list_of_Points=[]
    for traj in orig_list_of_Points:
        compressed_list_of_Points.append(bottom_up(traj,max_error))
    # print('OPW_batch ',len(orig_list_of_Points),' have done')
    return compressed_list_of_Points



# result = bottom_up(max_error)


#
# points=[]
# # print(os.getcwd())
# # print(sys.argv[0])
# # print('open?')
#
# file=open('Sample.txt','r')
# # file=open('../Zhou/Sample.txt','r')
#
# # print('open!')
# for n,line in enumerate(file):
#     if not line:
#         break
#     li=line.split(" ")
#     point=Point(float(li[0]),float(li[1]),float(li[2].strip()))
#     point.latlon2cartesian()
#     points.append(point)
#
# import argparse
#
# parser = argparse.ArgumentParser()
# parser.add_argument("args", help="describe the args of the algs ,just for record ,need to change " \
#                                  "in the specific file, like '90'")
# argps = parser.parse_args()
# max_error = int(eval(argps.args))#150#$#eval(input('max_error default 0.7') or '0.7')
# # print(max_error)
# result = bottom_up(max_error)
#
# fd=open(r"bottom-up_output.txt",'w')
# for i in range(len(result)):
#     fd.write("{} {} {}\n".format(result[i].point._x,result[i].point._y,int(result[i].point._t)))
# fd.close()
# print("压缩比：{}".format(len(result)/len(points)))





# import utils_copy

import os
from TrajCompress.src import basic_notion

def get_sample():
    orig_list_of_Points = []  
    this_traj=[]
    script_dir = os.path.dirname(os.path.abspath(__file__))
    print(f'you are getting sample from {script_dir}, using __file__:{__file__}')
    file_path = os.path.join(script_dir, '..', 'assets', 'Sample.txt')
    with open(file_path) as f:
        for line in f:
            a,o,t=list(map(eval,line.split()))
            this_traj.append(basic_notion.Point.Point(a,o,t))
    orig_list_of_Points.append(this_traj)
    orig_list_of_Points.append(this_traj[:20])
    return orig_list_of_Points